#include "derivative.h" /* include peripheral declarations */
#include "main.h"
#include "Accel.h"
#include "IIC.h"
#define ACCELADDRESS 0x1D

extern volatile char IICData[6];

/****************************************************************
*AccelWrite    Used to write to the Accelorometer
*
*Input:  addr - Address of register to write to 
*        data - Data to be written to register
*
*Output: 0 - free
*        1 - busy try again later        
****************************************************************/
int AccelWrite(int reg, int data){      
  return(IICOut(ACCELADDRESS, reg, data));
}

/****************************************************************
*AccelRead     Used to read from the Accelorometer
*
*Input:  addr - Address of register to read from 
*
*Output: 0 - free
*        1 - busy try again later
*        data returned in array IICData[]
****************************************************************/
int AccelRead(int StartReg, int ByteCount){
  return(IICIn(ACCELADDRESS, StartReg, ByteCount));
}
  
/****************************************************************
*AccelGrabData     Used to read acceloration in the x, y, and z 
*                  directions
*
*Input:  *PtX - Pointer to X variable
*        *PtY - Pointer to Y variable
*        *PtZ - Pointer to Z variable
*
*Output: None
****************************************************************/
void AccelGrabData(signed char* PtX, signed char* PtY, signed char* PtZ){
  while(AccelRead(0x06, 3) == 1);   
  *PtX = IICData[0];      
  *PtY = IICData[1];    
  *PtZ = IICData[2];
}

/****************************************************************
*AccelInit    This will initiallize the Accel
*
*Input:  None
*
*Output: None       
****************************************************************/

//don't use till updated

void AccelInit(void){
  //(void)AccelWrite(0x18, 0xB8);    //Control 1 - 125 Hz filter, no XYZ values
  //(void)AccelWrite(0x19, 0x04);    //Control 2 - strong drive on SDA
  //(void)AccelWrite(0x19, 0x22);  //Pulse detect
  //WaitUs(20);
  //(void)AccelWrite(0x18, 0x02);  //Interupt on Int1   
  //WaitUs(20);
  //(void)AccelWrite(0x1B, 0x40);  //Tap threshold level  
  //WaitUs(20);
  //(void)AccelWrite(0x1C, 0x10);  //Longest duration time  
  //WaitUs(20);
  //(void)AccelWrite(0x16, 0x05);//default mode, 2g
  //(void)AccelWrite(0x16, 0x03);  //pulse detect mode, 8g
  
  (void)AccelWrite(0x16, 0x05);//default mode, 2g
WaitUs(20);  
  (void)AccelWrite(0x16, 0x15);//default mode, 2g self-test
WaitUs(40);  
  (void)AccelWrite(0x16, 0x05);//default mode, 2g  
WaitUs(20);    
}

 //***This section give the address of important internal registers
 //***of the accel, and what value should be written there
 //                         
 //X Out 8bit       0x06          X data
 //Y Out 8bit       0x07          Y data
 //Z Out 8bit       0x08          Z data
 //Status           0x09, 0x01    data ready bit (can be on INT1 pin)
 //
 //Mode control     0x16, 0x05    Data ready bit relected on INT1 pin
 //                               SPI 4 wire mode (don't care)
 //                               Self test enable/disable
 //                               8g 4g 2g select
 //                               Mode select
 //Control 1        0x18, 0xB8    Digital filter band 62.5Hz or 125Hz
 //                               X, Y, Z enable disable
 //Control 2        0x19, 0x04    ICC pins drive strength (optional)