#include "derivative.h" /* include peripheral declarations */
#include "main.h"
#include "LCD.h"

/****************************************************************
*LCDInit     Initialized the LCD Screen
*
*Input:  None
*Output: None
*Notes:  /RESET = PortF_Bit7
*        /CS = PortG_Bit2
*        A0  = PortG_Bit3
*        E   = PortG_Bit4
*        /RW = PortG_Bit5
****************************************************************/  
void LCDInit(void){  
//Send start up procedure to LCD
  PTFD_PTFD7 = 0;             //Set Reset = Low
  WaitUs(2000);
  PTFD_PTFD7 = 1;             //Set Reset = High
  WaitUs(20000);
  PTGDD = 0x3C;               //Set PortG LCD pins to Outputs
  LCDDataOut(LCD_CMD,0xA0);      //Set segment driver direction 0 = Normal, 1 = Reverse (this will mirror the display)
  LCDDataOut(LCD_CMD,0xAE);      //Turn Display OFF  
  LCDDataOut(LCD_CMD,0xC8);      //Set scan direction of COM output terminal 0 = (COM0-63), 1 = (COM63-0)
  LCDDataOut(LCD_CMD,0xA2);      //Set the voltage bias ratio for LCD 0 = 1/9, 1 = 1/7  
  LCDDataOut(LCD_CMD,0x2F);      //Set power supply circuit functions (0 = OFF, 1 = ON) DB2 = Booster Circuit, DB1 = Voltage Regulator Circuit, DB0 = Voltage Follower Circuit
  LCDDataOut(LCD_CMD,0x26);      //Set internal resistor ratio  
  LCDDataOut(LCD_CMD,0x81);      //Allow for LCD Brightness to be adjusted, must be followed by LCD brightness (0x3F - 0)  
  LCDDataOut(LCD_CMD,0x10);      //Set LCD Brightness (0x3F - 0)                   
  LCDDataOut(LCD_CMD,0xAF);      //Turn Display ON 
}

/****************************************************************
*LCDDataOut     Send pure data to the LCD screen
*
*Input:  data1        command code
*        data2        data bus   
*         
*Output: None
*Notes:  Lower two bits of data1 are A0 and /RW respectively.
*        Lower 8 bits of data2 are the data bus
*        /CS = PortG_Bit2
*        A0  = PortG_Bit3
*        E   = PortG_Bit4
*        /RW = PortG_Bit5
****************************************************************/  
void LCDDataOut(int data1, int data2){  
  //Send Data to LCD 
  PTGD_PTGD3 = (LCD_DATA & data1) ? 1 : 0;  //Set A0 = 0 for command, = 1 for data
  PTGD_PTGD5 = 0;                         //Set /RW
  PTGD_PTGD2 = 0;                         //Set Chip Select low                  
  PTGD_PTGD4 = 1;                         //Set E High
  PTCD = (char)data2;                     //Write data to DB7-0
  __asm("nop"); 
  PTGD_PTGD4 = 0;                         //Set E Low
  PTGD_PTGD5 = 1;                         //Set /RW high 
  __asm("nop");    
  PTGD_PTGD2 = 1;                         //Set Chip Select high 
  __asm("nop");           
} 

/****************************************************************
*LCDSetCursorPos     Set position for cursor
*
*Input:  page            page to place cursor (0-8), 8 is just one line at bottom of LCD
*        column          column to place cursor (0-127)
*Output: None
****************************************************************/ 
void LCDSetCursorPos(int page, int column){
  //Set output location 
  LCDDataOut(LCD_CMD,(char)(0x10 | ((0xF0 & column)/16)));    //Set Column Address Upper
  LCDDataOut(LCD_CMD,(char)(0x0F & column));                  //Set Column Address Lower   
  LCDDataOut(LCD_CMD,(char)(0xB0 | (0x0F & page)));           //Set Page Address  
}
