#include "derivative.h" /* include peripheral declarations */
#include "main.h"
#include "LCD.h"

/****************************************************************
*LCDInit     Initialized the LCD Screen
*
*Input:  None
*Output: None
*Notes:  /RESET = PortF_Bit7
*        /CS = PortG_Bit2
*        A0  = PortG_Bit3
*        E   = PortG_Bit4
*        /RW = PortG_Bit5
****************************************************************/  
void LCDInit(void){  
//Send start up procedure to LCD
  PTGD = 0x30;                //Set E = High, /RW = High, Chip select = Low, A0 = Low
  PTFD = 0x80;                //Set Reset = High
  PTFDD_PTFDD7 = 1;           //  
  PTFD_PTFD7 = 0;             //Set Reset = Low
  WaitUs(2000);
  PTFD_PTFD7 = 1;             //Set Reset = High
  WaitUs(20000);
  PTGDD = 0x3C;               //Set PortG LCD pertainant pins to Outputs
  LCDDataOut(LCD_CMD,0xA2);      //Set the voltage bias ratio for LCD 0 = 1/9, 1 = 1/7
  LCDDataOut(LCD_CMD,0xA0);      //Set segment driver direction 0 = Normal, 1 = Reverse (this will mirror the display)
  LCDDataOut(LCD_CMD,0xC8);      //Set scan direction of COM output terminal 0 = (COM0-63), 1 = (COM63-0)
  LCDDataOut(LCD_CMD,0xA4);      //Allows for all pixles to be turned on 0 = Normal display, 1 = All pixles ON
  LCDDataOut(LCD_CMD,0x40);      //Specify the display start line address (This sets which horizontal line in ram is the top line on the LCD.  By changing this dynamically the screen can scroll up and down.)
  LCDDataOut(LCD_CMD,0x25);      //Set internal resistor ratio
  LCDDataOut(LCD_CMD,0x81);      //Allow for LCD Brightness to be adjusted, must be followed by LCD brightness (0x3F - 0)
  LCDDataOut(LCD_CMD,0x07);      //Set LCD Brightness (0x3F - 0)                 
  LCDDataOut(LCD_CMD,0x2F);      //Set power supply circuit functions (0 = OFF, 1 = ON) DB2 = Booster Circuit, DB1 = Voltage Regulator Circuit, DB0 = Voltage Follower Circuit
  LCDDataOut(LCD_CMD,0xAF);      //Turn Display ON
}

/****************************************************************
*LCDDataOut     Send pure data to the LCD screen
*
*Input:  data1        command code
*        data2        data bus   
*         
*Output: None
*Notes:  Lower two bits of data1 are A0 and /RW respectively.
*        Lower 8 bits of data2 are the data bus
*        /CS = PortG_Bit2
*        A0  = PortG_Bit3
*        E   = PortG_Bit4
*        /RW = PortG_Bit5
****************************************************************/  
void LCDDataOut(int data1, int data2){  
  //Send Data to LCD 
  PTGD_PTGD3 = (0x0002 & data1)/2;        //Set A0
  PTGD_PTGD5 = (0x0001 & data1);          //Set /RW
  PTGD_PTGD2 = 0;                         //Set Chip Select low                  
  PTGD_PTGD4 = 1;                         //Set E High
  PTCD = (char)data2;                     //Write data to DB7-0
  PTCDD = 0xFF;                           //Set Data Direction 1 = OUTPUT  
  PTGD_PTGD4 = 0;                         //Set E Low
  PTGD_PTGD2 = 1;                         //Set Chip Select high
  PTCDD = 0x00;                           //Set Data Direction to input           
} 

/****************************************************************
*LCDCursor     Turn cursor on or off
*
*Input:  data     0 off
*                 1 on & blinking fast
*                 2 on & blinking slowly
*                 3 on steady
*Output: None
****************************************************************/ 
void LCDCursor(int data){ 
  if(data == 0x00) {
    LCDDataOut(LCD_CMD,0xAC);          //Turn OFF Cursor Blink
  }
  else {
    LCDDataOut(LCD_CMD,0xAD);          //Allow Cursor Blink
    LCDDataOut(LCD_CMD,data);          //Set Cursor Blink Type
  }    
}

/****************************************************************
*LCDSetCursorPos     Set position for cursor
*
*Input:  page            page to place cursor (0-8), 8 is just one line at bottom of LCD
*        column          column to place cursor (0-127)
*Output: None
****************************************************************/ 
void LCDSetCursorPos(int page, int column){
  //Set output location 
  LCDDataOut(LCD_CMD,(char)(0xB0 | (0x0F & page)));           //Set Page Address
  LCDDataOut(LCD_CMD,(char)(0x10 | ((0xF0 & column)/16)));    //Set Column Address Upper
  LCDDataOut(LCD_CMD,(char)(0x0F & column));                  //Set Column Address Lower   
}