#include "derivative.h" /* include peripheral declarations */
#include "main.h"
#include "LCD.h"

extern const char CharacterLibrary[97][6];
extern const char minus[4][12];
extern const char period[4][8];
extern const char zero[4][21];
extern const char one[4][21];
extern const char two[4][21];
extern const char three[4][21];
extern const char four[4][21];
extern const char five[4][21];
extern const char six[4][21];
extern const char seven[4][21];
extern const char eight[4][21];
extern const char nine[4][21];
extern volatile char LCDString[33];
extern volatile float FixedFloat;
extern volatile long FixedLong;
extern volatile int FixedInt;
extern volatile char FixedChar;

/****************************************************************
*LCDCharOut     Put one character on the LCD
*
*Input:  data          character to be displayed (ascii)
*Output: None
****************************************************************/ 
void LCDCharOut(int data){
  int i;  
  for(i=0;i<6;i++)                                    //Write char to LCD  
    LCDDataOut(LCD_DATA,CharacterLibrary[data-32][i]);     //
} 

/****************************************************************
*LCDStringOut     Update LCD based on 
*
*Input:  stringpt      String pointer (ascii)
*Output: None
*
*Note:  This function writes a string to the LCD screen starting 
*       at a certain address and going until the end of the string
****************************************************************/ 
void LCDStringOut(const char* stringpt){
  char sglChar,k;
  sglChar = *stringpt;
  
  while(sglChar!=0) {                                      //Loop through pointer data for individual Char    
    for(k=0;k<6;k++) {                                     //Loop to output Char Hex data
      LCDDataOut(LCD_DATA,CharacterLibrary[sglChar-32][k]);  //Write char to LCD
    }
    stringpt++;
    sglChar = *stringpt;
  }    
}

/****************************************************************
*LCDBlank     Clear all data Ram in LCD
*
*Input:  None
*Output: None
****************************************************************/
void LCDBlank(void){
  int i, j;
  for(j=0;j<=8;j++){    
    LCDSetCursorPos(j,0);
    for(i=0;i<128;i++){    
      LCDDataOut(LCD_DATA, 0);
    }
  }
}

/****************************************************************
*LCDPageBlank     Clear all data Ram on one page in LCD
*
*Input:  page  -  Page to be blanked (0-7)
*Output: None
****************************************************************/
void LCDPageBlank(int page){
  int i;      
  LCDSetCursorPos(page,0);
  for(i=0;i<128;i++){    
    LCDDataOut(LCD_DATA, 0);
  }  
}

/****************************************************************
*LCDSectionBlank     Clear all data Ram of one section in LCD
*
*Input:  page  -  Page section is on (0-7)
*        start -  column to start blanking (0-128)
*        stop  -  column to stop blanking on (0-128)
*Output: None
****************************************************************/
void LCDSectionBlank(int page, int start, int stop){          
  if(start < stop){    
    LCDSetCursorPos(page,start);    
    for(;start<=stop;start++)      
      LCDDataOut(LCD_DATA, 0);   
  }
  else{
    LCDSetCursorPos(page,stop);
    for(;stop<=start;stop++)
      LCDDataOut(LCD_DATA, 0);
  }  
}

/****************************************************************
*LCDLongIntToString     This takes data and turns it into a string
*                       of numbers
*
*Input:  data             (long int)
*        dest             (char*)
*        MaxStringLength  The maximum number of character that can be displayed (int)
*        divider          This is for fractions.  desired number is data/(10^divider)(0-10)
*Output: length           Length of new string (int)
*
*Note: The resulting string is put into the given destination.
*      This function only works for integers
****************************************************************/ 
int LCDLongIntToString(long int data, char* dest, int MaxStringLength, int divider){
  int i, sign, length;
  char tempvec[13];
  
  if(divider<0 || divider>10){           //Check that divider is withing range
    *dest = '#';                         //
    *(dest+1) = 0;                       //
    return(1);                           //
  }                                      //
    
  sign = 0;                              //set sign and make sure data is positive
  if(data<0){                            //
    data = 0 - data;                     //
    sign = 1;                            //
  }                                      //
  
  if(data==0){                           //account for zero
    i = 0;                               //
    if(divider > 0){                     //account for decimal zero
      while(divider > 0){                //
        tempvec[i] = '0';                //
        divider--;                       //
        i++;                             //
      }                                  //
      tempvec[i] = '.';                  //
      i++;                               //
    }                                    //
    tempvec[i] = '0';                    //
    length = i+1;                        //
    if(length > MaxStringLength){        //if tring is too long print pound sign
      *dest = '#';                       //
      *(dest+1) = 0;                     //
      return(1);                         //
    }                                    //
    for(;i>=0;i--)                       //reverse and print to destination
      *(dest+length-i-1) = tempvec[i];   //
    *(dest+length) = 0;                  //                             
    return(length);                      //
  }                                      //
      
  i = 0;                                 //turn non-zero number to string  
  while(data>0){                         //
    tempvec[i] = (data%10)+'0';          //
    data /= 10;                          //    
    i++;                                 //    
    divider--;                           //
    if(divider == 0){                    //add decimal if needed
      tempvec[i] = '.';                  //      
      i++;                               //
    }                                    //
  }                                      //  
  
  if(divider>0){                         //
    while(divider > 0){                  //add placeholding zeros if needed    
      tempvec[i] = '0';                  //
      i++;                               //
      divider--;                         //        
    }                                    //
    tempvec[i] = '.';                    //add decimal for numbers with zeros on right of decimal
    i++;                                 //    
  }                                      //
  if(divider == 0){                      //add zero for decimals betwen 1 and -1
    tempvec[i] = '0';                    //    
    i++;                                 //
  }                                      //
  
  length = i;
  if(sign){                              //add sign if needed
    tempvec[i] = '-';                    //      
    length++;                            //    
  }                                      //  
  if(length>MaxStringLength){            //if string is too large then set as pound sign
    i = 0;                               //
    if(sign){                            //
      *dest = '-';                       //
      i++;                               //
    }                                    //
    *(dest+i) = '#';                     //
    i++;                                 //
    *(dest+i) = 0;                       //
    length = i;                          //
  }                                      //
  else{                                  //
    for(i=0;i<length;i++)                //reverse string and print to destination
      *(dest+i) = tempvec[length-i-1];   //                                     
    *(dest+i) = 0;                       //add null terminator
  }                                      //
                                           
  return(length);                        
}

/****************************************************************
*LCDFloatToString     This takes data and turns it into a string
*                     of numbers
*
*Input:  data             (float)
*        dest             (char*)
*        MaxStringLength  The maximum number of character that can be displayed (int)
*        Precision        Number of digits wanted to the right of the decimal point (int)
*Output: length           Length of new string (int)
*
*Note: The resulting string is put into the given destination.
*      This function accounts for all possible IEEE Standard 754
*      Floating Point Numbers.
*      Precision must be positive, and care must be taken to make sure
*      the destination string is long enough.         
*      Resulting string is null terminated.
****************************************************************/ 
int LCDFloatToString(float data, char* dest, int MaxStringLength, int Precision){
  int i, j, k, length; 
  long bits, templongbits, templong1;
  int exp, Base2Exp, Base10Exp, offset;
  char sign, tempvec[20], TempVec[20];      
   
  FixedFloat = data;
  asm{
    LDA 0x81             //Float to Long (bits)
    STA 0x85             //
    LDA 0x82             //
    STA 0x86             //
    LDA 0x83             //
    STA 0x87             //
    LDA 0x80             //Float to Int (exp)
    STA 0x88             //
    LDA 0x81             //
    STA 0x89             //
    LDA 0x80             //Float to Char (sign)
    STA 0x8A             //
  }  
  bits = FixedLong & 0x007FFFFF;
  exp = ((FixedInt & 0x7F80) / 0x80) - 127; 
  sign = (FixedChar & 0x80) >> 7;                      //sign=1 means negative, sign=0 means positive
   
  if(Precision+2+sign>MaxStringLength)                 //Keep precision within bounds of string length
    Precision = MaxStringLength - (2+sign);            //  
  if(Precision<0)                                      //No need for negative precision
    Precision = 0;                                     // 
   
  if(exp==-127){
    //if(bits==0){
      // Zero
      *dest = '0';
      if(Precision!=0){
        *(dest+1) = '.';
        for(i=0;i<Precision;i++)
          *(dest+2+i) = '0';            
        *(dest+2+i) = 0;
        length = Precision + 2;
      } 
      else{
        *(dest+1) = 0;
        length = 1;      
      }
    
    //  we display zero for denormilized anyways so the if else is pointless
    //} 
    //else{
      //De-Normalized      
      //***************still needed*****************
      //just make it read Zero because this is way small for our application
      //*dest = '0';
      //*(dest+1) = '.';
      //for(i=0;i<Precision;i++)
      //  *(dest+2+i) = '0';            
      //*(dest+2+i) = 0;
      //length = Precision + 2;
      //********************************************            
    //}
    //*/
    
  } 
  else if(exp==128){
    if(bits==0){
      //Infinity
      *dest = 0xF3;
      *(dest+1) = 0;
      length = 1;
    } 
    else{
      //NaN
      *dest = 'N';
      *(dest+1) = 'a';
      *(dest+2) = 'N';
      *(dest+3) = 0;      
      length = 3;
    }
  } 
  else{
    //Normalized
    bits += 0x800000;                                  //add implied bit
    templongbits = bits;                               //First get it as a base 10 number    
    Base2Exp = exp - 23;                               //subtract from Base2Exp to accomodate for the location of the decimal point
    Base10Exp = 0;                                     //                                                     
    if(Base2Exp>=0){                                   //If exp is positive        
      while(Base2Exp>0){                               //
        if((templongbits & 0xC0000000) == 0){          //
          Base2Exp--;                                  //
          templongbits *= 2;                           //
        }                                              //
        else{                                          //
          Base10Exp++;                                 //
          templongbits /= 10;                          //
        }                                              //
      }                                                //
    }                                                  //
    else{                                              //If exp is negative
      while(Base2Exp<0){                               //
        if((templongbits & 0xF8000000) == 0){          //
          Base10Exp--;                                 //
          templongbits *= 10;                          //
        }                                              //
        else{                                          //
          Base2Exp++;                                  //
          templongbits /= 2;                           //
        }                                              //
      }                                                //
    }                                                  //The number is now in base 10 with templongbits the mansa, Base10Exp the exponent, and sign the sign
                                                       //there is no decimal point in the mansa
    templong1 = 1;                                     //find offset that is needed for Base10Exp to reflect the number with one digit in front of the decimal
    offset = 0;                                        //offset + 1 = # of digits in mansa
    i = 0;                                             //
    while(i==0){                                       //
      templong1 *= 10;                                 //
      if(templongbits>=templong1)                      //
        offset++;                                      //
      else                                             //
        i = 1;                                         //
    }                                                  //Base10Exp+offset now reflects the number with 1 digit in front of the decimal
        
    i = offset + 1 + Base10Exp;                        //
    if(i<0)                                            //
      i = 1;                                           //
    i = i + sign + Precision;                          //
    if(Precision>0)                                    //
      i++;                                             //i is now the upper limit of the strings capacity
    
    j = 0;                                             //
    if((offset+Base10Exp+1)<=0){                       //
      j = 0 - (offset+Base10Exp+1);                    //
      j = j + sign + 2 + Precision;                    //
    }                                                  //j is now the lower limit of the strings ability to display a number
        
    if((i<=MaxStringLength) && (j<=MaxStringLength)){//Normal decimal representation
      //convert number to string
      i = 0;                                           //
      j = 0;                                           //
      k = 0;                                           //
      while(i==0 && j<20){                             //
        tempvec[j] = (templongbits % 10) + '0';        //
        templongbits /= 10;                            //                
        j++;                                           //        
        if(templongbits==0)                            //
          i = 1;                                       //                                      
      }                                                //j is now the string length (number of digits) and the string is backwards      
      
      //add any zeros to the right or left that're needed to be a valid number
      if(Base10Exp<=(0-j)){//zeros on the left
        i = (0-Base10Exp)-j+1;                         //determin how many zeros to add
        for(;i>0;i--){                                 //add them
          tempvec[j] = '0';                            //
          j++;                                         //
        }                                              //
      }
      else if(Base10Exp>0){//zeros on the right
        for(i=0;i<j;i++){                              //flip string
          TempVec[i] = tempvec[j-1-i];                 //
        }                                              //                
        for(;Base10Exp>0;Base10Exp--){                 //add zeros
          TempVec[j] = '0';                            //
          j++;                                         //          
        }                                              //
        for(i=0;i<j;i++){                              //flip string back
          tempvec[i] = TempVec[j-1-i];                 //
        }                                              //
      } 
      //else  //decimal lies in middle or right of mansa                  
            
      //add zeros needed for precision or take away decimals for precision and round
      if((0-Base10Exp)<Precision){  //add more zeros
        for(i=0;i<j;i++){                              //flip string
          TempVec[i] = tempvec[j-1-i];                 //
        }                                              //                
        for(;(0-Base10Exp)<Precision;Base10Exp--){      //add zeros
          TempVec[j] = '0';                            //
          j++;                                         //                    
        }                                              //
        for(i=0;i<j;i++){                              //flip string back
          tempvec[i] = TempVec[j-1-i];                 //
        }                                              //
      }      
      else if((0-Base10Exp)>Precision){//take precision away and round              
        for(i=0;i<j;i++){                              //flip string
          TempVec[i] = tempvec[j-1-i];                 //
        }                                              //                                
        while(Base10Exp<0-Precision){                  //
          j--;                                         //
          Base10Exp++;                                 //
        }                                              //                                        
        i = j;                                         //
        if(TempVec[i]>='5'){                           //if we need to round up then add and add carry until no carry.                                                         
          i = j-1;                                     //Add extra digit to left of string if needed.
          k = 1;                                       //
          while(k==1 && i>0){                          //            
            TempVec[i]++;                              //
            if(TempVec[i]==':')                        //
              TempVec[i] = '0';                        //
            else                                       //
              k = 0;                                   //
            i--;                                       //
          }                                            //
          if(k==1){                                    //add extra digit to left of string if needed
            for(i=j;i>0;i--)                           //
              TempVec[i] = TempVec[i-1];               //
            TempVec[0] = '1';                          //
          }                                            //
        }                                              //
        for(i=0;i<j;i++){                              //flip string back
          tempvec[i] = TempVec[j-1-i];                 //
        }                                              //                                           
      }
      //else    //perfect precision
                              
      //add negative sign if needed
      if(sign==1){     //add negative sign
        tempvec[j] = '-';                              //
        j++;                                           //
      }                                                //
      //else  //no sign
      
      
      for(i=0;i<j;i++){                                //flip string
        TempVec[i] = tempvec[j-1-i];                   //
      }                                                //                              
      
      
      //add decimal if needed
      if(Precision>0){     //add decimal        
        for(i=0;i<(0-Base10Exp);i++){                  //shift digits 
          TempVec[j-i] = TempVec[j-i-1];               //
        }                                              //
        TempVec[j-i]  = '.';                           //add decimal point
        j++;                                           //
      }                                          
     
     
      //add null terminator
      TempVec[j] = 0;                                  //add null terminator                                                     //
      for(i=0;i<=j;i++){                               //output string
        *(dest+i) = TempVec[i];                        //
      }                                                //
      length = j;                                      //set length
    }                                                  //
    
    
    else if((Base10Exp+offset<=99) && (Base10Exp+offset>=-99)){//Scientific notation                                                      
      Base10Exp += offset;                             //adjust exponent for scientific notation
      j = Precision;                                   //Remember previous precision in case we need to round to zero
      if(Precision+7>MaxStringLength)                  //keep precision within limits allowed by string length and scientific notation
        Precision = MaxStringLength - 7;               //
      if(Precision<1){                                 //if string is too short for exponent then say zero or infinity
        if(Base10Exp>0){                               //Infinity
          k = 0;                                       //
          if(sign==1){                                 //add negative if negative infinity
            *dest = '-';                               //
            k++;                                       //
          }                                            //
          *(dest+k) = 0xF3;                            //
          *(dest+1+k) = 0;                             //
          length = 1+k;                                //          
        }                                              //
        else{                                          //
          *dest = '0';                                 //
          *(dest+1) = '.';                             //
          for(i=0;i<j;i++)                             //
            *(dest+2+i) = '0';                         //
          *(dest+2+i) = 0;                             //
          length = j + 2;                              //
        }                                              //
        return(length);                                //
      }
      
                                                       //Otherwise number is good for scientific notation      
      i = 0;                                           //convert number to string
      j = 0;                                           //
      k = 0;                                           //
      while(i==0 && j<20){                             //
        tempvec[j] = (templongbits % 10) + '0';        //
        templongbits /= 10;                            //                
        j++;                                           //        
        if(templongbits==0)                            //
          i = 1;                                       //                                      
      }                                                //j is now the string length (number of digits) and the string is backwards      
      
      tempvec[j] = tempvec[j-1];                       //add decimal point
      tempvec[j-1] = '.';                              //
      j++;                                             //      
      if(sign==1){                                     //add negative sign is needed
        tempvec[j] = '-';                              //
        j++;                                           //
        k = 1;                                         //
      }                                                //      
      for(i=0;i<j;i++){                                //flip the string so it reads forward
        TempVec[i] = tempvec[j-i-1];                   //
      }                                                //everything to the right of the mansa should be done now
      if(j-k-2<Precision){                             //add zeros for precision if needed
        while(j-k-2<Precision){                        //
          TempVec[j] = '0';                            //
          j++;                                         //
        }                                              //
      }                                                //
      else if(j-k-2>Precision){                        //if needed take some precision away
        while(j-k-2>Precision){                        //
          j--;                                         //
        }                                              //        
      }                                                //
      TempVec[j] = ' ';                                //add space
      j++;                                             //
      if(Base10Exp<0){                                 //add sign for exponent
        TempVec[j] = '-';                              //and make exponent a positive number for string conversion
        j++;                                           //
        Base10Exp = 0 - Base10Exp;                     //
      }                                                //
      else{                                            //
        TempVec[j] = ' ';                              //
        j++;                                           //
      }                                                //      
      TempVec[j+1] = (Base10Exp%10) + '0';             //convert exponent to string
      Base10Exp /= 10;                                 //
      TempVec[j] = (Base10Exp%10) + '0';               //
      j = j + 2;                                       //
      TempVec[j] = 0;                                  //add null terminator
      length = j;                                      //set length to j
      for(i=0;i<=j;i++){                               //output string to destination
        *(dest+i) = TempVec[i];                        //
      }                                                //          
    }                                                
    else{                                              //Too big or too small
      if(Base10Exp>99){                                //If too big just say infinity
        *dest = 0xF3;                                  //
        *(dest+1) = 0;                                 //
        length = 1;                                    //
      }                                                //
      else if(Base10Exp<-99){                          //If too small just say zero
        *dest = '0';                                   //
        *(dest+1) = 0;                                 //
        length = 1;                                    //
      }  
    }                                         
  }    
  return(length);
}

/****************************************************************
*LCDStringToFloat     Turn a number from LCDSring into a float value
*
*Input:  dataPt       pointer to destination float 
*        InString     pointer to input string
*        PrecisionPt  pointer to return precision of inputed string
*Output: Error message
*
*Note:  If the string isn't a real number then this function
*       returns 0.
****************************************************************/ 
int LCDStringToFloat(float* dataPt, char* InString, int* PrecisionPt){
  int i, j, k, sign;
  int result=0, precision=0;             
  char tempvec[20], TempVec[20];
                                
  j = 0;                                               //Find length of string (j)
  while(*(InString+j)!=0)                              //
    j++;                                               //                                                      
  if(j>19)                                             //
    j = 19;                                            //
  
  //Remove spaces to right and left
  i = 0;                                               //Remove spaces to right
  while(i==0){                                         //
    if(*(InString+j-1)==' ')                           //
      j--;                                             //                                                      
    else                                               //
      i = 1;                                           //                                                       
  }                                                    //
  for(i=0;i<j;i++)                                     //flip string    
    tempvec[i] = *(InString+j-1-i);                    //       
  i = 0;                                               //Remove spaces to left
  while(i==0){                                         //
    if(tempvec[j-1]==' ')                              //
      j--;                                             //                                                      
    else                                               //
      i = 1;                                           //                                                       
  }                                                    //      
  //Check for valid formating
  if(tempvec[j-1]=='-'){//Look for sign.              
    sign = 1;                                          //sign=1 means negative, sign=0 means positive
    j--;                                               //
  }                                                    //
  else                                                 //
    sign = 0;                                          //
  k = 0;//search for decimal and garbage in string
  for(i=0;i<j;i++){                                    //
    if(tempvec[i]=='.')                                //
      k++;                                             //
    else if((tempvec[i]<'0') || (tempvec[i]>'9'))      //
      k = k + 10;                                      //
  }                                                    //
  if(k==0 || k==1){//if there's no garbage carry on  
    for(i=0;i<j;i++)                                   //flip string so it's forward
      TempVec[i] = tempvec[j-1-i];                     //  
    if(k==1){                                          //Determin number of times to divide by 10 (k)
      i = 0;                                           //if there's no decimal point then there is no dividing by 10 and k should be zero
      k = 0;                                           //
      while(k<j && i==0){                              //
        if(TempVec[j-k-1]=='.')                        //                                               
          i = 1;                                       //    
        else                                           //
          k++;                                         //
      }                                                //
    }                                                  //        
    *PrecisionPt = k;//Return precision (# of digits right of the decimal)        
    *dataPt = 0;//Convert string to float
    for(i=0;i<j;i++){                                  //
      if(TempVec[i]!='.'){                             //                                           
        *dataPt *= 10;                                 //
        *dataPt += TempVec[i] - '0';                   //
      }                                                //
    }                                                  //
    for(i=0;i<k;i++)                                   //scale by factor 
      *dataPt /= 10;                                   //
    if(sign)                                           //make negative if needed
      *dataPt *= -1;                                   //
    result = 1;                                        //set result to reflect no errors
  }                                          
//  else if(){//check for scientific notation    
//  }         
  else{//string was total garbage
    *dataPt = 0;                                       //
    *PrecisionPt = 0;
    result = 0;                                        //  
  }                                                    //
  return(result);                                      //  
} 

/****************************************************************
*LCDBigstringOut     Put string on LCD in large font
*
*Input:  startPage   - Starting LCD page (0-7)
*        startColumn - starting LCD column (0-127)
*        data        - string to be displayed (ascii)
*Output: None
****************************************************************/ 
void LCDBigStringOut(int startPage, int startColumn, char* data){
  int i, k, pageOffset, charWidth; 
  char* Pt;  
  for(pageOffset=0;pageOffset<4;pageOffset++){
    LCDSetCursorPos(startPage+pageOffset, startColumn); 
    k = 0;
    while(*(data+k) != 0){                    //run til end of string    
      if(*(data+k) == '-'){                     //decide length and section of library for output
        charWidth = 12;                         //
        Pt = (unsigned char*)&minus[pageOffset][0];             //
      }                                         //
      else if(*(data+k) == '.'){                //
        charWidth = 8;                          //
        Pt = (unsigned char*)&period[pageOffset][0];            //
      }                                         //
      else{                                     //
        charWidth = 21;                         //
        switch(*(data+k)){                      //
          case '0':                             //
            Pt = (unsigned char*)&zero[pageOffset][0];          //
            break;                              //
          case '1':                             //
            Pt = (unsigned char*)&one[pageOffset][0];           //
            break;                              //
          case '2':                             //
            Pt = (unsigned char*)&two[pageOffset][0];           //
            break;                              //
          case '3':                             //
            Pt = (unsigned char*)&three[pageOffset][0];         //
            break;                              //
          case '4':                             //
            Pt = (unsigned char*)&four[pageOffset][0];          //
            break;                              //
          case '5':                             //
            Pt = (unsigned char*)&five[pageOffset][0];          //
            break;                              //
          case '6':                             //
            Pt = (unsigned char*)&six[pageOffset][0];           //
            break;                              //
          case '7':                             //
            Pt = (unsigned char*)&seven[pageOffset][0];         //
            break;                              //
          case '8':                             //
            Pt = (unsigned char*)&eight[pageOffset][0];         //
            break;                              //
          default:                              //
            Pt = (unsigned char*)&nine[pageOffset][0];          //
            break;                              //
        }                                       //
      }                                         
      for(i=0;i<charWidth;i++)                  //Write char to LCD  
        LCDDataOut(LCD_DATA,*(Pt+i));             //
      k++;    
    }
  }  
} 

/****************************************************************
*LCDLengthForBigFont     determins number of columns used to 
*                        diplay string in big font
*
*Input:  data   - string to be displayed (ascii)
*Output: length - columns needed
****************************************************************/ 
int LCDLengthForBigFont(char* data){
  int i=0, length=0;
  while(*(data+i) != 0){
    if(*(data+i) == '-')
      length += 12;
    else if(*(data+i) == '.')
      length += 8;
    else  
      length += 21;
    i++;
  }
  return(length);
}