#include "derivative.h" /* include peripheral declarations */
#include "main.h"
#include "LCD.h"
#include "Menu.h"
#include "Accel.h"
#include "eeprom.h"
#include "OtherBlowByStuff.h"
#include <string.h>
#define USERINPUTSTRINGLENGTH 8
#define EEPROM_SIGNATURE  0x5a

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <time.h>

//Variable Declaration
void (*menu_ptr)(char) = menu_BlowBy_Default;

//Backlight
unsigned char backLightTime;
extern short backlightctr;
unsigned char backLightMode;

//Power off
unsigned char inactivePwrOffTime;
extern short inactivepwroffctr;
unsigned char lowBatPwrOffTime;
extern short lowbatpwroffctr;

//User Input
float *UserInputFloatPt, UserInputFloat, UserInputDivider;
int *UserInputIntPt, UserInputInt;
int UserInputPrecision, UserInputMult; 
char UserInputString[USERINPUTSTRINGLENGTH+1];
long int UserInputLongInt, UserInputHigh, UserInputLow;
extern volatile char ButtonHeld;
extern unsigned char options_enabled;

//Blowby specific
extern float K1, DK1, UEK1, K2, DK2, UEK2, UnitsConversion, OutFactor, FAn2Volts;
extern char UnitsLabel, menuArrow;
unsigned char meterRating;
unsigned char filterAverage;
unsigned char includeRPM;
unsigned char pulsesPerRev2;

//Average
extern volatile int AverageOnOff;
//Square Root
extern volatile int SquareRootOnOff;

//Recording
unsigned char recordingMode;      // 1 = enabled
unsigned char recordStartStop;    // 1 = recording

short playbackCurrRec, playbackHigh, playbackLow;

// RTC
extern volatile time_t seconds;
extern time_t safeseconds;

//Constants
const char SpNull[1] = {""};
const char SpCFM[4] = {"CFM"};
const char SpLPS[4] = {"LPS"};
const char SpOn[3] = {"On"};
const char SpOff[4] = {"Off"};
const char SpOptions[8] = {"Options"};
const char SpRezero[7] = {"Rezero"};
const char SpBack[5] = {"Back"};
const char SpSelect[7] = {"Select"};
const char SpResetDefaults[15] = {"Reset Defaults"};
const char SpMeterRating[13] = {"Meter Rating"};
const char SpUnits[6] = {"Units"};
const char SpFilter[7] = {"Filter"};
const char SpAdvancedCal[13] = {"Advanced Cal"};
const char SpBacklight[14] = {"Backlight/Pwr"};
const char SpBacklightOn[15] = {"Backlt On With"};
const char SpBacklight1[7] = {"Button"};
const char SpBacklight2[7] = {"Motion"};
const char SpBacklight3[14] = {"Button/Motion"};
const char SpBacklightTime[15] = {"Backlight Time"};
const char SpInactivePwrOff[15] = {"InactivePwrOff"};
const char SpLowBatPwrOff[15] = {"Low Bat PwrOff"};
const char SpNever[7] = {" never"};
const char SpMinutes[5] = {" min"};
const char SpSeconds[5] = {" sec"};
const char SpNone[5] = {"None"};
const char SpSecAverage[13] = {" Sec Average"};
const char SpUEK1[5] = {"UEK1"};
const char SpUEK2[5] = {"UEK2"};
const char SpSquareRoot[12] = {"Square Root"};
const char SpDisplayThres[12] = {"Dsply Thres"};
const char SpOutputFactor[14] = {"Output Factor"};
const char SpResetDefaultsL1[18] = {"Reseting Defaults"};
const char SpResetDefaultsL2[20] = {"will return readout"};
const char SpResetDefaultsL3[20] = {"to original factory"}; 
const char SpResetDefaultsL4[15] = {"settings.  OK?"};
const char SpYes[4] = {"Yes"};
const char SpNo[3] = {"No"};
const char SpRezeroL1[18] = {"Press Button when"};
const char SpRezeroL2[12] = {"at No Flow."};
const char SpWait[5] = {"Wait"};
const char SpRecording[10] = {"Recording"};
const char SpRecordingMode[15] = {"Recording Mode"};
const char SpRecordingMode1[13] = {"Recording On"};
const char SpRecordingMode2[14] = {"Recording Off"};
const char SpRecordingRate[15] = {"Recording Rate"};
const char SpRecordingRateL1a[18] = {"Filtering = None."};
const char SpRecordingRateL1[18] = {"Recording is done"};
const char SpRecordingRateL2[20] = {"at the Filter rate,"};
const char SpRecordingRateL3a[11] = {"currently "};
const char SpRecordingRateL3b[8] = {" second"};
const char SpRecordingRateL3c[16] = {"at 0.1 second"};
const char SpPressAnyKey[15] = {"(Press any key"};
const char SpToContinue[15] = {" to continue.)"};
const char SpRecordingPlayback[9] = {"Playback"};
const char SpRecordingErase[11] = {"Erase Data"};
const char SpRecordingEraseL1[15] = {"Erase All Data"};
const char SpRecordingEraseL2[4] = {"OK?"};
const char SpRecordingTimeDate[10] = {"Time/Date"};
const char SpRecordingTimeDateL1[16] = {"Use PC software"};
const char SpRecordingTimeDateL2[19] = {"to reset time/date"};                                                              
const char SpRecordingRPM[12] = {"Include RPM"};
const char SpRecordingPulsesRev[11] = {"Pulses/Rev"};
const char SpSavingOptions[18] = {"Saving options..."};
const char SpRecordStart[13] = {"Start Rec."};
const char SpRecordStop[13] = {" Stop Rec."};

const char SpRecordingFullL1[13] = {"Memory full."};
const char SpRecordingFullL2[19] = {"Go into Options to"};
const char SpRecordingFullL3[20] = {"playback/erase data"};
const char SpNoData[8] = {"No Data"};
const char SpRPM[4] = {"RPM"};

const float DK1_Table[4] = {1.67, 3.34, 6.67, 10};

void LCDBlank17()     // clear all but top line of LCD
{
  unsigned char i;
  for (i=1; i < 8; i++)
  {
      LCDPageBlank(i);    
  }
}

struct EEPROMoptions_struct
{
  unsigned char signature;
  unsigned char backLightMode;
  unsigned char backLightTime;
  unsigned char meterRating;
  char unitsLabel;
  unsigned char filterAverage;
  int squareRootOnOff;
  float UEK1;
  float UEK2;
  float displayThres;
  float outFactor;
  unsigned char recordingMode;
  unsigned char inactivePwrOffTime;  
  unsigned char lowBatPwrOffTime;
  unsigned char signature2;       // marks beginning of new data added
  unsigned char options_enabled;
  unsigned char includeRPM;
  unsigned char pulsesPerRev2;
};

struct EEPROMoptions_struct EEPROMoptions;

void updateEEPROMoptions()
{
  EEPROMoptions.signature = EEPROM_SIGNATURE;
  EEPROMoptions.backLightMode = backLightMode;
  EEPROMoptions.backLightTime = backLightTime;
  EEPROMoptions.meterRating = meterRating;
  EEPROMoptions.unitsLabel = UnitsLabel;
  EEPROMoptions.filterAverage = filterAverage;
  EEPROMoptions.squareRootOnOff = SquareRootOnOff;
  EEPROMoptions.UEK1 = UEK1;
  EEPROMoptions.UEK2 = UEK2;
  EEPROMoptions.displayThres = DisplayThres;
  EEPROMoptions.outFactor = OutFactor;
  EEPROMoptions.recordingMode = recordingMode;   
  EEPROMoptions.inactivePwrOffTime = inactivePwrOffTime;
  EEPROMoptions.lowBatPwrOffTime = lowBatPwrOffTime;
  EEPROMoptions.signature2 = EEPROM_SIGNATURE;
  EEPROMoptions.options_enabled = options_enabled;
  EEPROMoptions.includeRPM = includeRPM;
  EEPROMoptions.pulsesPerRev2 = pulsesPerRev2;
}

void updateEEPROMfromoptions(unsigned char flag)
{
  updateEEPROMoptions();
  
  if (flag)
  {   
    LCDBlank17();
    LCDSetCursorPos(3,10);    
    LCDStringOut(SpSavingOptions);
  }
  (void)EraseEEPROMPage((unsigned char*)EEPROM_BASE,((sizeof(EEPROMoptions)-1)/8)+1);   // 8 bytes per page
  (void)ProgEEPROM((unsigned char*)EEPROM_BASE, (unsigned char*)&EEPROMoptions, sizeof(EEPROMoptions));       
  if (flag)
  {
    LCDPageBlank(3);
  }  
}

void readoptionsfromEEPROM()
{
  (void)memcpy(&EEPROMoptions,(unsigned char*)EEPROM_BASE,sizeof(EEPROMoptions));
  if (EEPROMoptions.signature==EEPROM_SIGNATURE)
  {  
    backLightMode = EEPROMoptions.backLightMode;
    backLightTime = EEPROMoptions.backLightTime;
    meterRating = EEPROMoptions.meterRating;
    UnitsLabel = EEPROMoptions.unitsLabel;     
    filterAverage = EEPROMoptions.filterAverage;    
    SquareRootOnOff = EEPROMoptions.squareRootOnOff;
    UEK1 = EEPROMoptions.UEK1;
    UEK2 = EEPROMoptions.UEK2;
    DisplayThres = EEPROMoptions.displayThres;
    OutFactor = EEPROMoptions.outFactor;
    recordingMode = EEPROMoptions.recordingMode;
    inactivePwrOffTime = EEPROMoptions.inactivePwrOffTime;
    lowBatPwrOffTime = EEPROMoptions.lowBatPwrOffTime;
    
    if (EEPROMoptions.signature2==EEPROM_SIGNATURE)
    { 
      options_enabled = EEPROMoptions.options_enabled;
      includeRPM = EEPROMoptions.includeRPM;
      pulsesPerRev2 = EEPROMoptions.pulsesPerRev2;   
    }
    else  // new EEPROM options, init to default values
    {
      options_enabled = 1;
      includeRPM = 2;       // off
      pulsesPerRev2 = 2;
      
      updateEEPROMfromoptions(0);   // first time, update EEPROM with new values         
    }
    
    // set derived values
    UnitsConversion = (UnitsLabel == 0) ? 1 : 0.47195;
    BlowByAverageOnOff(filterAverage);
    DK1 = DK1_Table[meterRating];
    DK2 = 1.05;
    K1 = DK1 * UEK1;
    K2 = DK2 + UEK2;           
  } 
  else
  {
    updateEEPROMfromoptions(0);   // first time, update EEPROM with default values
  }
}

void waitForRelease(unsigned char flag)
{
  if (flag)
  {
    Start16Hz();  
  }
  while(0xF0 & PTBD)
  {
    __RESET_WATCHDOG();  // feeds the dog        
  }  
  WaitUs(50000);  
}

void pressAnyKey()
{
  LCDSetCursorPos(6,20);
  LCDStringOut(SpPressAnyKey);          
  LCDSetCursorPos(7,20);
  LCDStringOut(SpToContinue);          
  waitForRelease(1);      // also call Start16Hz
  while((0xF0 & PTBD)==0)
  {
    __RESET_WATCHDOG();   // feeds the dog        
  }
  WaitUs(50000);
  waitForRelease(0);  
}

void powerOffandWaitForKey(unsigned char viabutton)
{
  if (viabutton==0)
  {
    Stop16Hz();   // if Off button pressed, already called  
  }
  LCDBlank();
  if (recordStartStop)
  {       
    (void)startStopRecording(0,0);  // stop recording
  }
  NBACKLIGHT = 1;           // backlight off
  if (viabutton)
  {    
    waitForRelease(0);   
  }
  Stop20Hz();
  menu_ptr = 0;             // so isr_Button will turn backlight back on if enabled
  Buttons = 0;
  
  disable_peripherals();
  while (1)
  {   
    asm{
       STOP
    }
    if (Buttons)            // o.w. assume RTC interrupt, do STOP again
    {
      break;  
    }
  }
  Buttons = 0;              // make sure we ignore this key
  enable_peripherals();
  
  Start20Hz();
  if (lowBatPwrOffTime)
  {
    lowbatpwroffctr = 4*lowBatPwrOffTime;         // lowBatPwrOffTime in seconds, decr 4 times/sec     
  }
  
  menuArrow = 0;
  menu_ptr = menu_BlowBy_Default; //set menu var
  MainScreen(1,1);          // also calls waitForRelease, Start16Hz
}

/****************************************************************
*All Menus          
*
*Input:  button - port B
*Output: None
****************************************************************/
void MainScreen(unsigned char wfrFlag,unsigned char s16Flag)
{
  LCDSetCursorPos(0,0);
  LCDStringOut(SpOptions);
  LCDSetCursorPos(7,0);
  LCDStringOut(SpRezero);
  LCDSetCursorPos(6,CFM_RIGHTX-20);
  if(UnitsLabel == 0)
    LCDStringOut(SpCFM);
  else
    LCDStringOut(SpLPS);
  LCDSetCursorPos(0,70);
  if (recordingMode==1)
  {   
    if (recordStartStop==0)
    {  
      LCDStringOut(SpRecordStart);  
    }
    else
    {
      LCDStringOut(SpRecordStop);     
    }
  }
  LCDSetCursorPos(7,109);
  LCDStringOut(SpOff);
  if (wfrFlag)
  {    
    waitForRelease(s16Flag);    // also call Start16Hz 
  }
  else if (s16Flag)
  {
    Start16Hz();  
  }
}
 
/****************************************************************
*default_menu          default menu 
*                  
*Note: 
****************************************************************/ 
void menu_BlowBy_Default(char button)
{  
char full;
  Stop16Hz();
  menuArrow = 1;            // inhibits big font update
  switch(button){
    case BUTTON1: 
      LCDOptionsMenuOut(1);
      LCDSectionBlank(0,0,41);
      LCDSetCursorPos(0,0);
      LCDStringOut(SpBack);
      menu_ptr = menu_BlowBy_Options;//set menu var    
      waitForRelease(1);    // also call Start16Hz                                     
      break;
    case BUTTON2:                                
      LCDBlank();               //rezero           
      LCDSetCursorPos(0,0);
      LCDStringOut(SpBack);  
      LCDSetCursorPos(2,0);
      LCDStringOut(SpRezeroL1);
      LCDSetCursorPos(3,0);
      LCDStringOut(SpRezeroL2); 
      menu_ptr = menu_BlowBy_Rezero;//set menu var
      waitForRelease(1);    // also call Start16Hz                                                                         
      break;
    case BUTTON3:             // record start/stop
      full = 0;
      if (recordingMode==1)
      {       
        full = startStopRecording(1 - recordStartStop,1);
      }
      waitForRelease(0);
      if (full)
      {
        LCDBlank17();
        LCDSectionBlank(0,0,41);
        LCDSetCursorPos(0,0);
        LCDStringOut(SpBack);        
        LCDSetCursorPos(2,10);
        LCDStringOut(SpRecordingFullL1);
        LCDSetCursorPos(3,10);
        LCDStringOut(SpRecordingFullL2);  
        LCDSetCursorPos(4,10);
        LCDStringOut(SpRecordingFullL3);
        pressAnyKey();        // also calls Start16Hz
        LCDBlank17();
        menuArrow = 0;        // re-enable update
        MainScreen(1,1);      // also calls waitForRelease, Start16Hz
        return;                           
      }
      menuArrow = 0;          // re-enable update
      waitForRelease(1);      // also call Start16Hz      
      break;     
    case BUTTON4:             // Off
      powerOffandWaitForKey(1);    
      break;                 
  }  
}

/****************************************************************
*menu_BlowBy_Rezero         
*
*Note: 
****************************************************************/ 
void menu_BlowBy_Rezero(char button){  
  char i;
  float FAn2Volts_=0;
  switch(button){           
    case BUTTON3:
    case BUTTON4:
      LCDBlank();
      LCDSetCursorPos(3,52);
      LCDStringOut(SpWait);  
      //do average here
      for(i=0;i<20;i++){
        Stop20Hz();
        FAn2Volts_ += FAn2Volts;      
        Start20Hz();
        WaitUs(50000);       // also resets the WDT
      }
      FAn2Volts_ /= 20;
      Stop20Hz();                   
      UEK2 = FAn2Volts_ - DK2;
      if(UEK2 > 5)
        UEK2 = 5;
      else if(UEK2 < -5)
        UEK2 = -5;
      K2 = DK2 + UEK2;
      numberReadings = 0;
      sumOfCFMReadings = 0.0;
      averageCFM = 0.0;       
      ReStart20Hz();
      // falls into       
    case BUTTON1: //back
      LCDBlank();
      menuArrow = 0;
      menu_ptr = menu_BlowBy_Default;//set menu var                                              
      MainScreen(1,1);     // also calls waitForRelease, Start16Hz;
      break;    
  }      
}

/****************************************************************
*menu_BlowBy_Options          
*
*Note: 
****************************************************************/ 
void menu_BlowBy_Options(char button) { 
  char i; 
  Stop16Hz();
  switch(button){
    case BUTTON1:     
      LCDBlank();
      menuArrow = 0;
      menu_ptr = menu_BlowBy_Default;//set menu var      
      MainScreen(1,1);       // also calls waitForRelease, Start16Hz      
      return;                // not break
    case BUTTON2:
      switch(menuArrow){
        case 1:                             //recording
          menuArrow = 1;        
          menu_BlowBy_RecordingMenu();                                         
          break;      
        case 2:                               //backlight
          menuArrow = 1;        
          menu_BlowBy_BacklightMenu();
          break;               
        case 3:                               //reset defaults
          LCDBlank17();
          LCDSetCursorPos(7,0);
          LCDStringOut(SpSelect);
          LCDSetCursorPos(1,0);
          LCDStringOut(SpResetDefaultsL1);
          LCDSetCursorPos(2,0);
          LCDStringOut(SpResetDefaultsL2);
          LCDSetCursorPos(3,0);
          LCDStringOut(SpResetDefaultsL3);
          LCDSetCursorPos(4,0);
          LCDStringOut(SpResetDefaultsL4);          
          LCDSetCursorPos(5,44);
          LCDStringOut(SpYes);       
          LCDSetCursorPos(6,44);
          LCDStringOut(SpNo);         
          menuArrow = 6;      
          LCDSetCursorPos(menuArrow,38);
          LCDCharOut(RT_ARROW);  //->  
          menu_ptr = menu_BlowBy_ResetDefaults;                 
          break;
        case 4:                               // meter rating
          LCDBlank17();
          LCDSetCursorPos(7,5);
          LCDStringOut(SpSelect);          
          LCDSetCursorPos(1,44);
          LCDCharOut('3');
          LCDCharOut(' ');
          LCDStringOut(SpCFM);
          LCDSetCursorPos(2,44);
          LCDCharOut('6');
          LCDCharOut(' ');
          LCDStringOut(SpCFM);
          LCDSetCursorPos(3,44);
          LCDCharOut('1');
          LCDCharOut('2');
          LCDCharOut(' ');
          LCDStringOut(SpCFM);
          LCDSetCursorPos(4,44);
          LCDCharOut('1');
          LCDCharOut('8');
          LCDCharOut(' ');
          LCDStringOut(SpCFM);
          menuArrow = 1;
          while((DK1_Table[menuArrow-1] < DK1) && (menuArrow<5))            
            menuArrow++;          
          LCDSetCursorPos(menuArrow,38);
          LCDCharOut(RT_ARROW);  //->                  
          menu_ptr = menu_BlowBy_MeterRating;//set menu var          
          break;
        case 5:                             // units
          LCDBlank17();                         
          LCDSetCursorPos(7,0);
          LCDStringOut(SpSelect);                                       
          LCDSetCursorPos(1,44);
          LCDStringOut(SpCFM);          
          LCDSetCursorPos(2,44);
          LCDStringOut(SpLPS); 
          menuArrow = UnitsLabel + 1;          
          LCDSetCursorPos(menuArrow,38);
          LCDCharOut(RT_ARROW);  //->           
          menu_ptr = menu_BlowBy_Units;     //set menu var               
          break;        
        case 6:                             // filter
          LCDBlank17();
          LCDSetCursorPos(7,0);
          LCDStringOut(SpSelect); 
          LCDSetCursorPos(1,44);          
          LCDStringOut(SpNone);
          for(i=2;i<=6;i++){
            LCDSetCursorPos(i,44);
            LCDCharOut(i+47);
            LCDStringOut(SpSecAverage);
          }                                                 
          menuArrow = (AverageOnOff/10)+1;
          LCDSetCursorPos(menuArrow,38);
          LCDCharOut(RT_ARROW);  //->                    
          menu_ptr = menu_BlowBy_Filter;       
          break;        
        case 7:                             // advanced cal
          LCDBlank17();
          LCDSetCursorPos(7,0);
          LCDStringOut(SpSelect); 
          LCDSetCursorPos(1,44);
          LCDStringOut(SpUEK1);
          LCDSetCursorPos(2,44);
          LCDStringOut(SpUEK2);
          LCDSetCursorPos(3,44);
          LCDStringOut(SpSquareRoot);
          LCDSetCursorPos(4,44);
          LCDStringOut(SpDisplayThres);                                                  
          LCDSetCursorPos(5,44);
          LCDStringOut(SpOutputFactor);                                                            
          menuArrow = 1;
          LCDSetCursorPos(1,38);
          LCDCharOut(RT_ARROW);  //->                                
          menu_ptr = menu_BlowBy_AdvancedCal;       
          break;        
      }
      waitForRelease(0);            
      break;
    case BUTTON3:
      if(menuArrow > 1){
        LCDSectionBlank(menuArrow,38,43);
        menuArrow--;
        LCDSetCursorPos(menuArrow,38);
        LCDCharOut(RT_ARROW);  //->  
      }
    break;
    case BUTTON4:
      if(menuArrow < 7){
        LCDSectionBlank(menuArrow,38,43);
        menuArrow++;
        LCDSetCursorPos(menuArrow,38);
        LCDCharOut(RT_ARROW);  //->  
      }
    break;  
  }  
  Start16Hz();
}                           

/****************************************************************
*menu_BlowBy_ResetDefaults          
*
*Note: 
****************************************************************/ 
void set_defaults(void)
{
  backLightMode = 1;
  backLightTime = 1; 
  meterRating = 1;
  UnitsLabel = 0;
  filterAverage = 0;       
  SquareRootOnOff=0;
  UEK1=1;
  UEK2=0;    
  DisplayThres = 0.1;
  OutFactor = 1;
  recordingMode = 2;      // 1 = enabled    
  inactivePwrOffTime = 5;
  lowBatPwrOffTime = 20;
  includeRPM = 2;
  pulsesPerRev2 = 2;
  // NOTE: does not restore options_enabled
  
  // set derived values
  Stop20Hz();
  UnitsConversion = (UnitsLabel == 0) ? 1 : 0.47195;
  BlowByAverageOnOff(filterAverage);
  DK1 = DK1_Table[meterRating];
  DK2 = 1.05;  
  K1 = DK1 * UEK1;
  K2 = DK2 + UEK2;
  averageCFM = 0.0;  
  ReStart20Hz();                                       
}

void menu_BlowBy_ResetDefaults(char button){ 
  Stop16Hz();
  switch(button){    
    case BUTTON2:   
      LCDSectionBlank(menuArrow,44,61);   //select
      WaitUs(60000);
      LCDSetCursorPos(menuArrow,44);
      if(menuArrow == 5)
      {
        set_defaults();
        updateEEPROMfromoptions(1);                
      } 
      // falls into 
    case BUTTON1:  
      menuArrow = 3;                      //back
      LCDOptionsMenuOut(menuArrow);  
      menu_ptr = menu_BlowBy_Options;//set menu var
      waitForRelease(0);                                               
      break;
    case BUTTON3:
      if(menuArrow > 5){             //up
        LCDSectionBlank(menuArrow,38,43);
        menuArrow--;
        LCDSetCursorPos(menuArrow,38);
        LCDCharOut(RT_ARROW);  //->  
      }
    break;
    case BUTTON4:
      if(menuArrow < 6){             //down
        LCDSectionBlank(menuArrow,38,43);
        menuArrow++;
        LCDSetCursorPos(menuArrow,38);
        LCDCharOut(RT_ARROW);  //->  
      }
    break;
  }
  Start16Hz();
}

/****************************************************************
*menu_BlowBy_Recording          
*
*Note: 
****************************************************************/
void menu_BlowBy_RecordingMenu()
{ 
  LCDBlank17();
  LCDSetCursorPos(7,0);
  LCDStringOut(SpSelect); 
  LCDSetCursorPos(1,44);
  LCDStringOut(SpRecordingMode);
  LCDSetCursorPos(2,44);
  LCDStringOut(SpRecordingRate); 
  LCDSetCursorPos(3,44);
  LCDStringOut(SpRecordingPlayback);
  LCDSetCursorPos(4,44);
  LCDStringOut(SpRecordingErase);
  LCDSetCursorPos(5,44);
  LCDStringOut(SpRecordingTimeDate);
  LCDSetCursorPos(6,44);
  LCDStringOut(SpRecordingRPM); 
  LCDSetCursorPos(7,44);
  LCDStringOut(SpRecordingPulsesRev);                                                                           
  LCDSetCursorPos(menuArrow,38);
  LCDCharOut(RT_ARROW);  //->                                
  menu_ptr = menu_BlowBy_Recording; 
}

void menu_BlowBy_Recording(char button)
{
  char i;
  struct tm * currtime;
  char timestr[26];
  time_t oldseconds;
       
  Stop16Hz();
  switch(button){
    case BUTTON1:      
      menuArrow = 1;                      //back
      LCDOptionsMenuOut(menuArrow);  
      menu_ptr = menu_BlowBy_Options;//set menu var
      waitForRelease(0);                                   
      break;
    case BUTTON2:  
      switch(menuArrow){              //Set up user input variables
        case 1:                             //recording mode
          LCDBlank17();
          LCDSetCursorPos(7,0);
          LCDStringOut(SpSelect); 
          LCDSetCursorPos(1,44);
          LCDStringOut(SpRecordingMode1);  
          LCDSetCursorPos(2,44);
          LCDStringOut(SpRecordingMode2);                                
          menuArrow = recordingMode;          
          LCDSetCursorPos(menuArrow,38);
          LCDCharOut(RT_ARROW);  //->  
          menu_ptr = menu_BlowBy_RecordingMode;       
          break;        
        case 2:                              //recording rate
          LCDBlank17();
          if (filterAverage)
          {
            LCDSetCursorPos(2,10);
            LCDStringOut(SpRecordingRateL1);
            LCDSetCursorPos(3,10);          
            LCDStringOut(SpRecordingRateL2);
            LCDSetCursorPos(4,10);                    
            LCDStringOut(SpRecordingRateL3a);
            LCDCharOut(filterAverage+'0');
            LCDStringOut(SpRecordingRateL3b);
          }
          else
          {
            LCDSetCursorPos(2,10);
            LCDStringOut(SpRecordingRateL1a);
            LCDSetCursorPos(3,10);          
            LCDStringOut(SpRecordingRateL1);
            LCDSetCursorPos(4,10);                             
            LCDStringOut(SpRecordingRateL3c); // 0.1 second for filterAverage = 0            
          }
          if (filterAverage!=1)
          {
            LCDCharOut('s');
          }
          LCDCharOut('.');
          pressAnyKey();        // also calls Start16Hz
          menu_BlowBy_RecordingMenu();
          waitForRelease(0);
          return;   // not break                                                                          
        case 3:                              //playback
          if (nextRecord)
          {            
            LCDBlank17();          
            playbackCurrRec = nextRecord;
            playbackHigh = nextRecord;
            playbackLow = 0;
            LCDSetCursorPos(4,18);
            LCDCharOut(RT_ARROW);  //->
            menu_ptr = menu_BlowBy_RecordPlayback;          
            LCDPlaybackOut();
          }
          else
          {
            LCDBlank17();
            LCDSetCursorPos(3,44);
            LCDStringOut(SpNoData);
            for (i=0; i < 20; i++)   // wait 1 sec
            {
              WaitUs(50000);          // also calls __RESET_WATCHDOG
            }                                  
            menu_BlowBy_RecordingMenu();                     
          }
          break; 
        case 4:                              //erase data
          LCDBlank17();
          LCDSetCursorPos(7,0);
          LCDStringOut(SpSelect);
          LCDSetCursorPos(2,20);
          LCDStringOut(SpRecordingEraseL1);
          LCDSetCursorPos(3,20);
          LCDStringOut(SpRecordingEraseL2);        
          LCDSetCursorPos(5,44);
          LCDStringOut(SpYes);       
          LCDSetCursorPos(6,44);
          LCDStringOut(SpNo);         
          menuArrow = 6;      
          LCDSetCursorPos(menuArrow,38);
          LCDCharOut(RT_ARROW);  //->  
          menu_ptr = menu_BlowBy_RecordErase;                                                                   
          break; 
        case 5:                              //time/date
          LCDBlank17();
          LCDSetCursorPos(2,10);                    

          asm("sei");
          safeseconds = seconds;      // disable ints around copying of seconds
          asm("cli");                      
          currtime = gmtime(&safeseconds); 
                
          if (currtime->tm_year>=100)
          {  
            (void)sprintf(timestr, "%02d:%02d:%02d %02d/%02d/%02d       ", 
            currtime->tm_hour,
            currtime->tm_min,  
            currtime->tm_sec,
            currtime->tm_mon+1,  
            currtime->tm_mday,
            (currtime->tm_year+YEAR0)%100);  
            
            LCDSetCursorPos(1,12);
            LCDStringOut(timestr);
          }
         
          LCDSetCursorPos(3,10);
          LCDStringOut(SpRecordingTimeDateL1);
          LCDSetCursorPos(4,10);          
          LCDStringOut(SpRecordingTimeDateL2);                  

          //modified pressAnyKey():
          LCDSetCursorPos(6,20);
          LCDStringOut(SpPressAnyKey);          
          LCDSetCursorPos(7,20);
          LCDStringOut(SpToContinue);          
          waitForRelease(1);      // also call Start16Hz
          
          oldseconds = 0;
          while((0xF0 & PTBD)==0)
          {
            asm("sei");
            safeseconds = seconds;      // disable ints around copying of seconds
            asm("cli");
                                          
            if (safeseconds != oldseconds)
            {              
              currtime = gmtime(&safeseconds);       
              if (currtime->tm_year>=100)
              {  
                (void)sprintf(timestr, "%02d:%02d:%02d %02d/%02d/%02d       ", 
                currtime->tm_hour,
                currtime->tm_min,  
                currtime->tm_sec,
                currtime->tm_mon+1,  
                currtime->tm_mday,
                (currtime->tm_year+YEAR0)%100);  
                
                LCDSetCursorPos(1,12);
                LCDStringOut(timestr);
                
                oldseconds = safeseconds;
              }
            }
          
            __RESET_WATCHDOG();   // feeds the dog        
          }
          WaitUs(50000);
          waitForRelease(0);           
                          
          menu_BlowBy_RecordingMenu();
          waitForRelease(0);
          return;   // not break                         
      case 6:                     //recording rpm 
        LCDBlank17();
        LCDSetCursorPos(1,44);
        LCDStringOut(SpYes);
        LCDSetCursorPos(2,44);
        LCDStringOut(SpNo);
        menuArrow = includeRPM;
        LCDSetCursorPos(menuArrow,38);
        LCDCharOut(RT_ARROW);  //->        
        menu_ptr = menu_BlowBy_RecordingRPM;   //set menu var
        break;      
      case 7:                     //recording pulses/rev       
        LCDBlank17();
        LCDSetCursorPos(7,0);
        LCDStringOut(SpSelect);           
        UserInputLongInt = 5*pulsesPerRev2;
        UserInputPrecision = 1;
        UserInputMult = 5;
        UserInputHigh = 160;
        UserInputLow = 5;
        for(i=1;i<=6;i++)
          LCDSectionBlank(i,15,127);
        LCDSetCursorPos(4,38);
        LCDCharOut(RT_ARROW);  //->
        menu_ptr = menu_BlowBy_UserInput2;          
        LCDUserInputOut(SpNull);                         
      }
      waitForRelease(0);
      break;
    case BUTTON3:
      if(menuArrow > 1){             //up
        LCDSectionBlank(menuArrow,38,43);
        menuArrow--;
        LCDSetCursorPos(menuArrow,38);
        LCDCharOut(RT_ARROW);  //->  
      }
    break;
    case BUTTON4:
      if(menuArrow < 7){             //down
        LCDSectionBlank(menuArrow,38,43);
        menuArrow++;
        LCDSetCursorPos(menuArrow,38);
        LCDCharOut(RT_ARROW);  //->  
      }
    break;
  }
  Start16Hz();
}

/****************************************************************
*menu_BlowBy_RecordingMode          
*
*Note: 
****************************************************************/ 
void menu_BlowBy_RecordingMode(char button){
  Stop16Hz();
  switch(button){    
    case BUTTON2:                   //select      
      recordingMode = menuArrow;
      updateEEPROMfromoptions(1);
      // falls into         
    case BUTTON1:     
      menuArrow = 1;                      //back
      menu_BlowBy_RecordingMenu();
      waitForRelease(0);                                
      break;
    case BUTTON3:
      if(menuArrow > 1){             //up
        LCDSectionBlank(menuArrow,38,43);
        menuArrow--;
        LCDSetCursorPos(menuArrow,38);
        LCDCharOut(RT_ARROW);  //->  
      }
      break;
    case BUTTON4:
      if(menuArrow < 2){             //down
        LCDSectionBlank(menuArrow,38,43);
        menuArrow++;
        LCDSetCursorPos(menuArrow,38);
        LCDCharOut(RT_ARROW);  //->  
      }
      break;
  }
  Start16Hz();
}

/****************************************************************
*menu_BlowBy_RecordPlayback         
*
*Note: 
****************************************************************/
void menu_BlowBy_RecordPlayback(char button)
{
  Stop16Hz();
  switch(button){    
    case BUTTON2:                       //select      
      break;
    case BUTTON1: 
      menuArrow = 2;                      //back
      menu_BlowBy_RecordingMenu();
      waitForRelease(0);                               
      break;    
    case BUTTON3:     
      //Up
      if(playbackCurrRec < playbackHigh){        
        if(ButtonHeld == 0)
          playbackCurrRec +=2;
        else if(ButtonHeld == 1)
          playbackCurrRec += 10;
        else
          playbackCurrRec += 100;      
        if(playbackCurrRec > playbackHigh)
         playbackCurrRec = playbackHigh;                                       
        LCDPlaybackOut();
      }
      break;
    case BUTTON4:
      //Down      
      if(playbackCurrRec > playbackLow){        
        if(ButtonHeld == 0)
          playbackCurrRec -=2;
        else if(ButtonHeld == 1)
          playbackCurrRec -= 10;
        else
          playbackCurrRec -= 100;      
        if(playbackCurrRec < playbackLow)
          playbackCurrRec = playbackLow;                                                                              
        LCDPlaybackOut();                
      }
      break;    
  }
  Start16Hz();
}

/****************************************************************
*menu_BlowBy_RecordErase         
*
*Note: 
****************************************************************/
void menu_BlowBy_RecordErase(char button)
{ 
  Stop16Hz();
  switch(button){    
    case BUTTON2:   
      LCDSectionBlank(menuArrow,44,61);   //select
      WaitUs(60000);
      LCDSetCursorPos(menuArrow,44);
      if(menuArrow == 5){
        init_recordData();                 
      }
      // falls into  
    case BUTTON1:  
      menuArrow = 4;                      //back
      menu_BlowBy_RecordingMenu(); 
      waitForRelease(0);                                           
      break;
    case BUTTON3:
      if(menuArrow > 5){             //up
        LCDSectionBlank(menuArrow,38,43);
        menuArrow--;
        LCDSetCursorPos(menuArrow,38);
        LCDCharOut(RT_ARROW);  //->  
      }
      break;
    case BUTTON4:
      if(menuArrow < 6){             //down
        LCDSectionBlank(menuArrow,38,43);
        menuArrow++;
        LCDSetCursorPos(menuArrow,38);
        LCDCharOut(RT_ARROW);  //->  
      }
      break;
  }
  Start16Hz();
}

/****************************************************************
*menu_BlowBy_RecordingRPM
*
*Note: 
****************************************************************/ 
void menu_BlowBy_RecordingRPM(char button){
  Stop16Hz();
  switch(button){
    case BUTTON2:
      LCDSectionBlank(menuArrow,44,62);   //select
      WaitUs(60000);
      LCDSetCursorPos(menuArrow,44);          
      Stop20Hz();
      includeRPM = menuArrow;
      updateEEPROMfromoptions(1);      
      ReStart20Hz();    
      // falls into    
    case BUTTON1:                                
      menuArrow = 6;                      //back
      menu_BlowBy_RecordingMenu();
      waitForRelease(0);                                
      break;
    case BUTTON3:
      if(menuArrow > 1){             //up
        LCDSectionBlank(menuArrow,38,43);
        menuArrow--;
        LCDSetCursorPos(menuArrow,38);
        LCDCharOut(RT_ARROW);  //->  
      }
      break;
    case BUTTON4:
      if(menuArrow < 2){             //down
        LCDSectionBlank(menuArrow,38,43);
        menuArrow++;
        LCDSetCursorPos(menuArrow,38);
        LCDCharOut(RT_ARROW);  //->  
      }
      break;
  }
  Start16Hz();
}

/****************************************************************
*menu_BlowBy_Backlight          
*
*Note: 
****************************************************************/
void menu_BlowBy_BacklightMenu() { 
  LCDBlank17();
  LCDSetCursorPos(7,0);
  LCDStringOut(SpSelect); 
  LCDSetCursorPos(1,44);
  LCDStringOut(SpBacklightOn);
  LCDSetCursorPos(2,44);
  LCDStringOut(SpBacklightTime);
  LCDSetCursorPos(3,44);
  LCDStringOut(SpInactivePwrOff);
  LCDSetCursorPos(4,44);
  LCDStringOut(SpLowBatPwrOff);                                                              
  LCDSetCursorPos(menuArrow,38);
  LCDCharOut(RT_ARROW);  //->                                
  menu_ptr = menu_BlowBy_Backlight;
}

void menu_BlowBy_Backlight(char button){
char i;   
  Stop16Hz();
  switch(button){
    case BUTTON1:      
      menuArrow = 2;                      //back
      LCDOptionsMenuOut(menuArrow);  
      menu_ptr = menu_BlowBy_Options;//set menu var
      waitForRelease(0);                                   
      break;
    case BUTTON2:  
      switch(menuArrow){              //Set up user input variables
        case 1:                             //backlight
          LCDBlank17(); 
          LCDSetCursorPos(7,0);
          LCDStringOut(SpSelect); 
          LCDSetCursorPos(1,44);
          LCDStringOut(SpBacklight1);  
          LCDSetCursorPos(2,44);
          LCDStringOut(SpBacklight2);
          LCDSetCursorPos(3,44);
          LCDStringOut(SpBacklight3);                                 
          menuArrow = backLightMode;          
          LCDSetCursorPos(menuArrow,38);
          LCDCharOut(RT_ARROW);  //->  
          menu_ptr = menu_BlowBy_BacklightOn;       
          break;        
        case 2:                              //backlight time
          LCDBlank17();
          LCDSetCursorPos(7,0);
          LCDStringOut(SpSelect);           
          UserInputLongInt = backLightTime;
          UserInputPrecision = 0;
          UserInputMult = 1;
          UserInputHigh = 99;
          UserInputLow = 1;
          for(i=1;i<=6;i++)
            LCDSectionBlank(i,15,127);
          LCDSetCursorPos(4,38);
          LCDCharOut(RT_ARROW);  //->
          menu_ptr = menu_BlowBy_BacklightTime;          
          LCDUserInputOut(SpMinutes);                                                          
          break;
        case 3:                              //inactive power off time
          LCDBlank17();
          LCDSetCursorPos(7,0);
          LCDStringOut(SpSelect);           
          UserInputLongInt = inactivePwrOffTime;
          UserInputPrecision = 0;
          UserInputMult = 5;
          UserInputHigh = 20;
          UserInputLow = 0;
          for(i=1;i<=6;i++)
            LCDSectionBlank(i,15,127);
          LCDSetCursorPos(4,38);
          LCDCharOut(RT_ARROW);  //->
          menu_ptr = menu_BlowBy_InactivePwrOffTime;          
          LCDUserInputOut(SpMinutes);                                                          
          break;
        case 4:                              //low battery power off
          LCDBlank17();
          LCDSetCursorPos(7,0);
          LCDStringOut(SpSelect);           
          UserInputLongInt = lowBatPwrOffTime;
          UserInputPrecision = 0;
          UserInputMult = 20;
          UserInputHigh = 120;
          UserInputLow = 0;
          for(i=1;i<=6;i++)
            LCDSectionBlank(i,15,127);
          LCDSetCursorPos(4,38);
          LCDCharOut(RT_ARROW);  //->
          menu_ptr = menu_BlowBy_LowBatPwrOffTime;          
          LCDUserInputOut(SpSeconds);                                                          
          break;                    
      }
      waitForRelease(0);
      break;
    case BUTTON3:
      if(menuArrow > 1){             //up
        LCDSectionBlank(menuArrow,38,43);
        menuArrow--;
        LCDSetCursorPos(menuArrow,38);
        LCDCharOut(RT_ARROW);  //->  
      }
      break;
    case BUTTON4:
      if(menuArrow < 4){             //down
        LCDSectionBlank(menuArrow,38,43);
        menuArrow++;
        LCDSetCursorPos(menuArrow,38);
        LCDCharOut(RT_ARROW);  //->  
      }
      break;
  }
  Start16Hz();
}

/****************************************************************
*menu_BlowBy_BacklightOn          
*
*Note: 
****************************************************************/ 
void menu_BlowBy_BacklightOn(char button){
  Stop16Hz();
  switch(button){    
    case BUTTON2:                   //select      
      backLightMode = menuArrow;
      updateEEPROMfromoptions(1);
      // falls into         
    case BUTTON1:     
      menuArrow = 1;                      //back
      menu_BlowBy_BacklightMenu();
      waitForRelease(0);                                
      break;
    case BUTTON3:
      if(menuArrow > 1){             //up
        LCDSectionBlank(menuArrow,38,43);
        menuArrow--;
        LCDSetCursorPos(menuArrow,38);
        LCDCharOut(RT_ARROW);  //->  
      }
      break;
    case BUTTON4:
      if(menuArrow < 3){             //down
        LCDSectionBlank(menuArrow,38,43);
        menuArrow++;
        LCDSetCursorPos(menuArrow,38);
        LCDCharOut(RT_ARROW);  //->  
      }
      break;
  }
  Start16Hz();
}

/****************************************************************
*menu_BlowBy_BacklightTime          
*
*Note: 
****************************************************************/
void menu_BlowBy_BacklightTime(char button)
{
  Stop16Hz();
  switch(button){    
    case BUTTON2:                       //select      
      backLightTime = (int)UserInputLongInt;
      backlightctr = 240*backLightTime;     
      updateEEPROMfromoptions(1);
      // falls into        
    case BUTTON1:  
      menuArrow = 2;                      //back
      menu_BlowBy_BacklightMenu();
      waitForRelease(0);                               
      break;    
    case BUTTON3:     
      //Up
      if(UserInputLongInt < UserInputHigh){        
        if(ButtonHeld == 0)
          UserInputLongInt++;
        else if(ButtonHeld == 1)
          UserInputLongInt += 10;
        else
          UserInputLongInt += 100;      
        if(UserInputLongInt > UserInputHigh)
          UserInputLongInt = UserInputHigh;                                       
        LCDUserInputOut(SpMinutes);
      }
      break;
    case BUTTON4:
      //Down      
      if(UserInputLongInt > UserInputLow){        
        if(ButtonHeld == 0)
          UserInputLongInt--;
        else if(ButtonHeld == 1)
          UserInputLongInt -= 10;
        else
          UserInputLongInt -= 100;      
        if(UserInputLongInt < UserInputLow)
          UserInputLongInt = UserInputLow;                                                                              
        LCDUserInputOut(SpMinutes);                
      }
      break;    
  }
  Start16Hz();
}

/****************************************************************
*menu_BlowBy_InactivePwrOffTime          
*
*Note: 
****************************************************************/
void menu_BlowBy_InactivePwrOffTime(char button)
{
  Stop16Hz();
  switch(button){    
    case BUTTON2:                       //select      
      inactivePwrOffTime = (int)UserInputLongInt;
      inactivepwroffctr = 240*inactivePwrOffTime;        
      updateEEPROMfromoptions(1);
      // falls into        
    case BUTTON1:  
      menuArrow = 3;                      //back
      menu_BlowBy_BacklightMenu();
      waitForRelease(0);                               
      break;    
    case BUTTON3:     
      //Up
      if(UserInputLongInt < UserInputHigh){        
        if(ButtonHeld == 0)
          UserInputLongInt += 5;
        else if(ButtonHeld == 1)
          UserInputLongInt += 10;
//        else
//          UserInputLongInt += 100;      
        if(UserInputLongInt > UserInputHigh)
          UserInputLongInt = UserInputHigh;                                       
        LCDUserInputOut(SpMinutes);
      }
      break;
    case BUTTON4:
      //Down      
      if(UserInputLongInt > UserInputLow){        
        if(ButtonHeld == 0)
          UserInputLongInt -= 5;
        else if(ButtonHeld == 1)
          UserInputLongInt -= 10;
//        else
//          UserInputLongInt -= 100;      
        if(UserInputLongInt < UserInputLow)
          UserInputLongInt = UserInputLow;                                                                              
        LCDUserInputOut(SpMinutes);                
      }
      break;    
  }
  Start16Hz();
}

/****************************************************************
*menu_BlowBy_LowBatPwrOffTime          
*
*Note: 
****************************************************************/
void menu_BlowBy_LowBatPwrOffTime(char button)
{
  Stop16Hz();
  switch(button){    
    case BUTTON2:                       //select      
      lowBatPwrOffTime = (int)UserInputLongInt;
      lowbatpwroffctr = 4*lowBatPwrOffTime;      
      updateEEPROMfromoptions(1);
      // falls into        
    case BUTTON1:  
      menuArrow = 4;                      //back
      menu_BlowBy_BacklightMenu();
      waitForRelease(0);                               
      break;    
    case BUTTON3:     
      //Up
      if(UserInputLongInt < UserInputHigh){        
        if(ButtonHeld == 0)
          UserInputLongInt += 20;
        else if(ButtonHeld == 1)
          UserInputLongInt += 40;
//        else
//          UserInputLongInt += 100;      
        if(UserInputLongInt > UserInputHigh)
          UserInputLongInt = UserInputHigh;                                       
        LCDUserInputOut(SpSeconds);
      }
      break;
    case BUTTON4:
      //Down      
      if(UserInputLongInt > UserInputLow){        
        if(ButtonHeld == 0)
          UserInputLongInt -= 20;
        else if(ButtonHeld == 1)
          UserInputLongInt -= 40;
//        else
//          UserInputLongInt -= 100;      
        if(UserInputLongInt < UserInputLow)
          UserInputLongInt = UserInputLow;                                                                              
        LCDUserInputOut(SpSeconds);                
      }
      break;    
  }
  Start16Hz();
}              
         
/****************************************************************
*menu_BlowBy_MeterRating          for seting the cfm rating, blow by
*
*Note: 
****************************************************************/ 
void menu_BlowBy_MeterRating(char button) {    
  Stop16Hz();
  switch(button){    
    case BUTTON2:                       
      LCDSectionBlank(menuArrow,44,79);        //select
      WaitUs(60000);
      LCDSetCursorPos(menuArrow,44);
      switch(menuArrow){
        case 1:
          LCDCharOut('3');
        break;
        case 2:
          LCDCharOut('6');
        break;
        case 3:
          LCDCharOut('1');
          LCDCharOut('2');
        break;
        case 4:
          LCDCharOut('1');
          LCDCharOut('8');
        break;
      } 
      LCDCharOut(' ');
      LCDStringOut(SpCFM);                 
      Stop20Hz();
      meterRating = menuArrow-1;
      updateEEPROMfromoptions(1);
      DK1 = DK1_Table[meterRating];
      K1 = DK1 * UEK1;
      ReStart20Hz(); 
      // falls intio   
    case BUTTON1:        
      menuArrow = 4;                           //back    
      LCDOptionsMenuOut(menuArrow);                     
      menu_ptr = menu_BlowBy_Options;//set menu var
      waitForRelease(0);                                       
      break;
    case BUTTON3:
      if(menuArrow > 1){             //up
        LCDSectionBlank(menuArrow,38,43);
        menuArrow--;
        LCDSetCursorPos(menuArrow,38);
        LCDCharOut(RT_ARROW);  //->  
      }
      break;    
    case BUTTON4:
      if(menuArrow < 4){             //down
        LCDSectionBlank(menuArrow,38,43);
        menuArrow++;
        LCDSetCursorPos(menuArrow,38);
        LCDCharOut(RT_ARROW);  //->  
      }
      break;
  }  
  Start16Hz();              
}

/****************************************************************
*menu_BlowBy_Units          for seting the units, blow by
*
*Note: 
****************************************************************/ 
void menu_BlowBy_Units(char button) {  
  Stop16Hz();
  switch(button){    
    case BUTTON2:      
      LCDSectionBlank(menuArrow,44,79);   //select
      WaitUs(60000);
      LCDSetCursorPos(menuArrow,44);
      if(menuArrow == 1)
        LCDStringOut(SpCFM);      
      else
        LCDStringOut(SpLPS);      
      Stop20Hz();
      if(menuArrow == 1){
        UnitsConversion = 1;
        UnitsLabel = 0;
      }
      else{
        UnitsConversion = .47195;
        UnitsLabel = 1;
      }
      updateEEPROMfromoptions(1);
      ReStart20Hz();
      // falls into        
    case BUTTON1:
      menuArrow = 5;                      //back
      LCDOptionsMenuOut(menuArrow);       
      menu_ptr = menu_BlowBy_Options;//set menu var
      waitForRelease(0);                                   
      break;
    case BUTTON3:
      if(menuArrow > 1){             //up
        LCDSectionBlank(menuArrow,38,43);
        menuArrow--;
        LCDSetCursorPos(menuArrow,38);
        LCDCharOut(RT_ARROW);  //->  
      }
      break;
    case BUTTON4:
      if(menuArrow < 2){             //down
        LCDSectionBlank(menuArrow,38,43);
        menuArrow++;
        LCDSetCursorPos(menuArrow,38);
        LCDCharOut(RT_ARROW);  //->  
      }
      break;
  }
  Start16Hz();
}

/****************************************************************
*menu_BlowBy_Filter
*
*Note: 
****************************************************************/ 
void menu_BlowBy_Filter(char button){
  Stop16Hz();
  switch(button){
    case BUTTON2:
      LCDSectionBlank(menuArrow,44,121);   //select
      WaitUs(60000);
      LCDSetCursorPos(menuArrow,44);     
      if(menuArrow == 1)
        LCDStringOut(SpNone);      
      else{        
        LCDCharOut(menuArrow+47);
        LCDStringOut(SpSecAverage);
      }
      Stop20Hz();
      if (recordStartStop)
      {       
        (void)startStopRecording(0,0);  // stop recording
      }      
      filterAverage = menuArrow-1; 
      updateEEPROMfromoptions(1);
      BlowByAverageOnOff(filterAverage);      
      ReStart20Hz();    
      // falls into    
    case BUTTON1: 
      menuArrow = 6;                      //back
      LCDOptionsMenuOut(menuArrow);  
      menu_ptr = menu_BlowBy_Options;//set menu var
      waitForRelease(0);                                   
      break;
    case BUTTON3:
      if(menuArrow > 1){             //up
        LCDSectionBlank(menuArrow,38,43);
        menuArrow--;
        LCDSetCursorPos(menuArrow,38);
        LCDCharOut(RT_ARROW);  //->  
      }
      break;
    case BUTTON4:
      if(menuArrow < 6){             //down
        LCDSectionBlank(menuArrow,38,43);
        menuArrow++;
        LCDSetCursorPos(menuArrow,38);
        LCDCharOut(RT_ARROW);  //->  
      }
      break;
  }
  Start16Hz();
}

/****************************************************************
*menu_BlowBy_AdvancedCal          
*
*Note: 
****************************************************************/ 
void menu_BlowBy_AdvancedCal(char button){  
int i;
  Stop16Hz();
  switch(button){
    case BUTTON1:      
      menuArrow = 7;                      //back
      LCDOptionsMenuOut(menuArrow);  
      menu_ptr = menu_BlowBy_Options;//set menu var 
      waitForRelease(0);                                  
      break;
    case BUTTON2:  
      UserInputDivider = 1;
      //possible variables 
      //UEK1 - 0.01
      //UEK2 - 0.01
      //SquarRootOnOff - na
      //Output factor - 0.01
      //Display Threshold high - 0.01
      //Display Threshold low - 0.01
      switch(menuArrow){              //Set up user input variables
        case 1://UEK1
          //UserInputFloat = UEK1;
          UserInputFloatPt = &UEK1;
          UserInputIntPt = 0;
          UserInputPrecision = 2;
          UserInputMult = 1;
          UserInputHigh = 400;              //user input high and low are off by a factor of 10^precision
          UserInputLow = 20;                //
         for(i=0;i<=UserInputPrecision;i++)
            UserInputDivider *= 10;
          UserInputLongInt = (long int)((*UserInputFloatPt)*UserInputDivider);  
          if(UserInputLongInt >= 0){
            if(UserInputLongInt%10 > 5)
              UserInputLongInt += 10;
          }
          else{
            if((-UserInputLongInt)%10 > 5)
              UserInputLongInt -= 10;           
          }           
          UserInputLongInt /= 10;
          UserInputDivider /= 10;     
        break;
        case 2://UEK2        
          //UserInputFloat = UEK2;
          UserInputFloatPt = &UEK2;
          UserInputIntPt = 0;
          UserInputPrecision = 2;
          UserInputMult = 1;
          UserInputHigh = 500;              //user input high and low are off by a factor of 10^precision
          UserInputLow = -500;              //
          for(i=0;i<=UserInputPrecision;i++)
            UserInputDivider *= 10;
          UserInputLongInt = (long int)((*UserInputFloatPt)*UserInputDivider);  
          if(UserInputLongInt >= 0){
            if(UserInputLongInt%10 > 5)
              UserInputLongInt += 10;
          }
          else{
            if((-UserInputLongInt)%10 > 5)
              UserInputLongInt -= 10;           
          }           
          UserInputLongInt /= 10;
          UserInputDivider /= 10;        
          break;
        //case 3://Square Root                                                                                        
        //break; //nothing to do here
        case 4://Display Thres                
          UserInputFloat = DisplayThres;
          UserInputFloatPt = &DisplayThres;
          UserInputIntPt = 0;
          UserInputPrecision = 2;
          UserInputMult = 1;
          UserInputHigh = 1000;            //user input high and low are off by a factor of 10^precision
          UserInputLow = 0;                //
          for(i=0;i<=UserInputPrecision;i++)
            UserInputDivider *= 10;
          UserInputLongInt = (long int)((*UserInputFloatPt)*UserInputDivider);  
          if(UserInputLongInt >= 0){
            if(UserInputLongInt%10 > 5)
              UserInputLongInt += 10;
          }
          else{
            if((-UserInputLongInt)%10 > 5)
              UserInputLongInt -= 10;           
          }           
          UserInputLongInt /= 10;
          UserInputDivider /= 10;
          break;        
        case 5://Output Factor        
          //UserInputFloat = OutFactor;
          UserInputFloatPt = &OutFactor;
          UserInputIntPt = 0;
          UserInputPrecision = 2;
          UserInputMult = 1;
          UserInputHigh = 400;            //user input high and low are off by a factor of 10^precision
          UserInputLow = 20;              //
          for(i=0;i<=UserInputPrecision;i++)
            UserInputDivider *= 10;
          UserInputLongInt = (long int)((*UserInputFloatPt)*UserInputDivider);  
          if(UserInputLongInt >= 0){
            if(UserInputLongInt%10 > 5)
              UserInputLongInt += 10;
          }
          else{
            if((-UserInputLongInt)%10 > 5)
              UserInputLongInt -= 10;           
          }           
          UserInputLongInt /= 10;
          UserInputDivider /= 10;                             
          break;      
      }       // end switch(menuArrow)
      //
      //maybe here again                                     
      //
      LCDSectionBlank(menuArrow,38,43);
      if(menuArrow!=3){
        for(i=1;i<=6;i++)
          LCDSectionBlank(i,44,127);
        LCDSetCursorPos(4,38);
        LCDCharOut(RT_ARROW);  //->
        LCDUserInputOut(SpNull);
        menu_ptr = menu_BlowBy_UserInput;   //set menu var
      }
      else{                                 //square root on off
        for(i=1;i<=6;i++)
          LCDSectionBlank(i,44,127);
        LCDSetCursorPos(1,44);
        LCDStringOut(SpOn);
        LCDSetCursorPos(2,44);
        LCDStringOut(SpOff);
        menuArrow = SquareRootOnOff + 1;
        LCDSetCursorPos(menuArrow,38);
        LCDCharOut(RT_ARROW);  //->        
        menu_ptr = menu_BlowBy_UserInputSquareRoot;   //set menu var
      }
      waitForRelease(0);
      break;
    case BUTTON3:
      if(menuArrow > 1){             //up
        LCDSectionBlank(menuArrow,38,43);
        menuArrow--;
        LCDSetCursorPos(menuArrow,38);
        LCDCharOut(RT_ARROW);  //->  
      }
      break;
    case BUTTON4:
      if(menuArrow < 5){             //down
        LCDSectionBlank(menuArrow,38,43);
        menuArrow++;
        LCDSetCursorPos(menuArrow,38);
        LCDCharOut(RT_ARROW);  //->  
      }
      break;
  }
  Start16Hz();
}

/****************************************************************
*menu_BlowBy_UserInputSquareRoot
*
*Note: 
****************************************************************/ 
void menu_BlowBy_UserInputSquareRoot(char button){
  Stop16Hz();
  switch(button){
    case BUTTON2:
      LCDSectionBlank(menuArrow,44,62);   //select
      WaitUs(60000);
      LCDSetCursorPos(menuArrow,44);           
      Stop20Hz();
      SquareRootOnOff = menuArrow - 1;
      updateEEPROMfromoptions(1);      
      ReStart20Hz();    
      // falls into    
    case BUTTON1: 
      LCDBlank17();
      LCDSetCursorPos(7,0);
      LCDStringOut(SpSelect);                               
      LCDSetCursorPos(1,44);              
      LCDStringOut(SpUEK1);
      LCDSetCursorPos(2,44);
      LCDStringOut(SpUEK2);
      LCDSetCursorPos(3,44);
      LCDStringOut(SpSquareRoot);
      LCDSetCursorPos(4,44);
      LCDStringOut(SpDisplayThres);                                                
      LCDSetCursorPos(5,44);
      LCDStringOut(SpOutputFactor);                                                            
      menuArrow = 3;
      LCDSetCursorPos(menuArrow,38);
      LCDCharOut(RT_ARROW);  //->                                
      menu_ptr = menu_BlowBy_AdvancedCal;
      waitForRelease(0);                                  
      break;
    case BUTTON3:
      if(menuArrow > 1){             //up
        LCDSectionBlank(menuArrow,38,43);
        menuArrow--;
        LCDSetCursorPos(menuArrow,38);
        LCDCharOut(RT_ARROW);  //->  
      }
      break;
    case BUTTON4:
      if(menuArrow < 2){             //down
        LCDSectionBlank(menuArrow,38,43);
        menuArrow++;
        LCDSetCursorPos(menuArrow,38);
        LCDCharOut(RT_ARROW);  //->  
      }
      break;
  }
  Start16Hz();
}

/****************************************************************
*menu_BlowBy_UserInput          
*
*Note:   *UserInputFloatPt     - variable to be changed
*        *UserInputIntPt       - variable to be changed.  one of these pointers must be zero
*        UserInputString       - String for LCD
*        UserInputStringLength - Length of string
*        UserInputPrecision    - # of digits to the right of decimal
*        UserInputMult         - multiplcation factor (e.g. 5 for 0, 5, 10, 15, 20 ...)
*        UserInputLongInt      - Used to help do math on screen as numbers are scrolled up and down
*        
****************************************************************/ 
void menu_BlowBy_UserInput(char button){  
  char i;    
  Stop16Hz();
  switch(button){    
    case BUTTON2:
      //select    
      *UserInputFloatPt = ((float)UserInputLongInt)/UserInputDivider;
      /*                                    
      Stop20Hz();           //update K1 and K2
      K1 = DK1 * UEK1;       //
      K2 = DK2 + UEK2;       //
      ReStart20Hz();          //                  
      */    
      // falls into    
    case BUTTON1:
      LCDSetCursorPos(4,38);  //back
      LCDCharOut(' ');
      for(i=1;i<=6;i++)                    
        LCDSectionBlank(i,44,110);              
      LCDSetCursorPos(1,44);              
      LCDStringOut(SpUEK1);
      LCDSetCursorPos(2,44);
      LCDStringOut(SpUEK2);                  
      LCDSetCursorPos(3,44);
      LCDStringOut(SpSquareRoot);
      LCDSetCursorPos(4,44);
      LCDStringOut(SpDisplayThres);                                                
      LCDSetCursorPos(5,44);
      LCDStringOut(SpOutputFactor); 
      LCDSetCursorPos(menuArrow,38);
      LCDCharOut(RT_ARROW);  //->                   
      menu_ptr = menu_BlowBy_AdvancedCal;//set menu var
      waitForRelease(0);                                   
      break;
    case BUTTON3:
      //Up
      if(UserInputLongInt < UserInputHigh){        
        if(ButtonHeld == 0)
          UserInputLongInt++;
        else if(ButtonHeld == 1)
          UserInputLongInt += 10;
        else
          UserInputLongInt += 100;      
        if(UserInputLongInt > UserInputHigh)
          UserInputLongInt = UserInputHigh;                                       
        LCDUserInputOut(SpNull);
      }
      break;
    case BUTTON4:
      //Down      
      if(UserInputLongInt > UserInputLow){        
        if(ButtonHeld == 0)
          UserInputLongInt--;
        else if(ButtonHeld == 1)
          UserInputLongInt -= 10;
        else
          UserInputLongInt -= 100;      
        if(UserInputLongInt < UserInputLow)
          UserInputLongInt = UserInputLow;                                                                              
        LCDUserInputOut(SpNull);                
      }      
      break;
  }
  Start16Hz();
}

void menu_BlowBy_UserInput2(char button)
{  
  Stop16Hz();
  switch(button){    
    case BUTTON2:                       //select      
      pulsesPerRev2 = (int)(2L*UserInputLongInt/10L);
      updateEEPROMfromoptions(1);
      // falls into        
    case BUTTON1:  
      menuArrow = 7;                      //back
      menu_BlowBy_RecordingMenu();
      waitForRelease(0);                               
      break;    
    case BUTTON3:     
      //Up
      if(UserInputLongInt < UserInputHigh){        
        if(ButtonHeld == 0)
          UserInputLongInt += 5;
        else if(ButtonHeld == 1)
          UserInputLongInt += 10;     
        if(UserInputLongInt > UserInputHigh)
          UserInputLongInt = UserInputHigh;                                       
        LCDUserInputOut(SpNull);
      }
      break;
    case BUTTON4:
      //Down      
      if(UserInputLongInt > UserInputLow){        
        if(ButtonHeld == 0)
          UserInputLongInt -= 5;
        else if(ButtonHeld == 1)
          UserInputLongInt -= 10;     
        if(UserInputLongInt < UserInputLow)
          UserInputLongInt = UserInputLow;                                                                              
        LCDUserInputOut(SpNull);                
      }
      break;    
  }
  Start16Hz();
}

/****************************************************************
*LCDOptionsMenuOut
*
*Input:  None
*OutPut: None
*       
*Note:  This does everything needed to transition to the options menu
*       Should only be called from a menu function.
****************************************************************/
void LCDOptionsMenuOut(char data){  
  LCDBlank17();
  LCDSetCursorPos(7,0);
  LCDStringOut(SpSelect);  
  LCDSetCursorPos(1,44);
  LCDStringOut(SpRecording);
  LCDSetCursorPos(2,44);
  LCDStringOut(SpBacklight);  
  LCDSetCursorPos(3,44);
  LCDStringOut(SpResetDefaults);
  LCDSetCursorPos(4,44);
  LCDStringOut(SpMeterRating);
  LCDSetCursorPos(5,44);
  LCDStringOut(SpUnits);
  LCDSetCursorPos(6,44);
  LCDStringOut(SpFilter);
  LCDSetCursorPos(7,44);
  LCDStringOut(SpAdvancedCal);                      
  LCDSetCursorPos(data,38);
  LCDCharOut(RT_ARROW);  //->           
}

/****************************************************************
*LCDUserInputOut
*
*Input:  None
*OutPut: None
*       
*Note:  This does everything needed to transition to the options menu
*       Should only be called from a menu function.
****************************************************************/
void LCDUserInputOut(const char* suffix){
  int i;
  //page 4
  LCDSectionBlank(4,44,110);  
  i = LCDLongIntToString(UserInputLongInt,UserInputString,USERINPUTSTRINGLENGTH,UserInputPrecision);                  
  LCDSetCursorPos(4,44);    
  LCDStringOut(UserInputString);  
  if ((suffix != SpNull) && (UserInputLongInt==0))    
  {
    LCDStringOut(SpNever);  
  }
  else
  {
    LCDStringOut(suffix);  
  }
  //page 5
  LCDSectionBlank(5,44,110);  
  if(UserInputLongInt > UserInputLow){
    i = LCDLongIntToString(UserInputLongInt-UserInputMult,UserInputString,USERINPUTSTRINGLENGTH,UserInputPrecision);
    LCDSetCursorPos(5,44);
    LCDStringOut(UserInputString);
    if ((suffix != SpNull) && (UserInputLongInt==1))    
    {
      LCDStringOut(SpNever);  
    }
    else
    {
      LCDStringOut(suffix);  
    }      
  }
  //page 6
  LCDSectionBlank(6,44,110);
  if(UserInputLongInt > UserInputLow+UserInputMult){
    i = LCDLongIntToString(UserInputLongInt-2*UserInputMult,UserInputString,USERINPUTSTRINGLENGTH,UserInputPrecision);
    LCDSetCursorPos(6,44);
    LCDStringOut(UserInputString);
    if ((suffix != SpNull) && (UserInputLongInt==2))    
    {
      LCDStringOut(SpNever);  
    }
    else
    {
      LCDStringOut(suffix);  
    }      
  }  
  //page 3
  LCDSectionBlank(3,44,110);  
  if(UserInputLongInt < UserInputHigh){
    i = LCDLongIntToString(UserInputLongInt+UserInputMult,UserInputString,USERINPUTSTRINGLENGTH,UserInputPrecision);
    LCDSetCursorPos(3,44);
    LCDStringOut(UserInputString);
    LCDStringOut(suffix);    
  }  
  //page 2
  LCDSectionBlank(2,44,110);  
  if(UserInputLongInt < UserInputHigh-UserInputMult){
    i = LCDLongIntToString(UserInputLongInt+2*UserInputMult,UserInputString,USERINPUTSTRINGLENGTH,UserInputPrecision);
    LCDSetCursorPos(2,44);
    LCDStringOut(UserInputString);
    LCDStringOut(suffix);    
  }           
}

void LCDPlaybackOut1(unsigned short i)
{
  char str [30];
  char* units;
  short data, tdata;
  short j;
  float rate, time, fData;
  struct tm currtime;
  
  for (j=i; j>=0; j--)
  {
    if (recordData[j]>=0x7ff0)
    {                
      tdata = recordData[j+1];
      currtime.tm_hour = (tdata >> 8) & 0xff;
      currtime.tm_min = tdata & 0xff;
      tdata = recordData[j+2]; 
      currtime.tm_sec = (tdata >> 8) & 0xff;
      currtime.tm_mon = (tdata & 0xff) - 1;
      tdata = recordData[j+3]; 
      currtime.tm_mday = (tdata >> 8) & 0xff;
      currtime.tm_year = (tdata & 0xff)+2000-YEAR0;                          
      break;      
    }
  }       
  
  data = recordData[j]&0x7;
  rate = (data) ? (float)data : 0.1;
  units = (recordData[j]&0x8) ? (char *)SpLPS : (char *)SpCFM;
      
  data = recordData[i];
  if ((i >= (j+1)) && (i <= (j+3)))
  {  
    if (i == (j+1))
    {
      (void)sprintf(str,"%02d:%02d:%02d         ",
        currtime.tm_hour,
        currtime.tm_min,  
        currtime.tm_sec);        
    }
    else if (i == (j+2))
    {
      (void)sprintf(str,"%02d/%02d/%02d         ",
        currtime.tm_mon+1,  
        currtime.tm_mday,
        (currtime.tm_year+YEAR0)%100);        
    }
    else if (i == (j+3))
    {     
      (void)sprintf(str,"                   ");    
    }
  }  
  else if (data >= 0x7ff0)      // marker
  {
    if (data == 0x7ffe)    // end
    {
      (void)sprintf(str," -------------    ");    
    }
    else
    {        
      (void)sprintf(str," -----%s-----    ",units);    
    }
  }
  else
  {
    time = rate*(i - (j+3))/2;
        
    fData = (float)data;
    fData = fData / 100.0;
    data = recordData[++i]; 
    (void)sprintf(str,"%7.1f:%6.2f %d   ",time,fData,data);    
  }
  LCDStringOut(str); 
}

void LCDPlaybackOut(void)
{
  unsigned short i;
 
  i = nextRecord - playbackCurrRec;   
   
  //page 4              
  LCDSetCursorPos(4,10);                // middle record
  LCDPlaybackOut1(i);
  //page 5
  LCDSetCursorPos(5,10);  
  if(playbackCurrRec > playbackLow){
    LCDPlaybackOut1(i+2);
  }
  else{   
    LCDSectionBlank(5,10,110);
  }
  //page 6
  LCDSetCursorPos(6,10);  
  if(playbackCurrRec > playbackLow+1){
    LCDPlaybackOut1(i+4);

  }
  else{   
    LCDSectionBlank(6,10,110);
  }  
  //page 3
  LCDSetCursorPos(3,10);
  if(playbackCurrRec < playbackHigh){
    LCDPlaybackOut1(i-2);

  }  
  else{  
    LCDSectionBlank(3,10,110);
  }
  //page 2
  LCDSetCursorPos(2,10);
  if(playbackCurrRec < playbackHigh-1){
    LCDPlaybackOut1(i-4);

  }
  else{  
    LCDSectionBlank(2,10,110);
  }            
}