#include "derivative.h" /* include peripheral declarations */
#include "main.h"
#include "menu.h"
#include "OtherBlowByStuff.h"

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <time.h>

float DisplayThres = 0.0;
volatile int AverageOnOff=0, SquareRootOnOff=0;
volatile unsigned short numberReadings;
volatile float sumOfCFMReadings = 0.0, averageCFM = 0.0;
extern char UnitsLabel;
extern unsigned char option_num;
extern char option_str[30];

extern volatile time_t seconds;
extern time_t safeseconds;

//record data
short recordData[RECORD_DATA_SIZE];
unsigned short nextRecord = 0, firstRecord = 0;

/****************************************************************
*addToRecordData     Initialize recording log to all 7FFF's
*
*Input:  none
*
*Output: none
*
*Note: Very specific to BlowBy app
****************************************************************/ 
void init_recordData()
{
  unsigned short i;
  
  (void)startStopRecording(0,menuArrow==0);   // stop recording
  
  for (i=0; i < RECORD_DATA_SIZE; i++)
  {
    recordData[i] = 0x7fff;   // unused value        
  }
  nextRecord = 0;
  firstRecord=0;  
  lastDumpCommand=0; 
}

/****************************************************************
*startStopRecording     Add data to the recording log
*
*Input:  mode         -  1 = start, 0 = stop
*        updateFlag   -  if 1, update MainScreen before exiting
*
*Output: none
*
*Note: Very specific to BlowBy app
****************************************************************/ 
char startStopRecording(unsigned char mode, unsigned updateFlag)
{
    if ((mode==0) && (recordingMode==1) && recordStartStop)
    {
        output4Hex((unsigned short) 0x7ffe);         // debug       
             
        recordData[nextRecord] = 0x7ffe;             // do not increment nextRecord
         
        recordStartStop = 0;
        
        firstRecord=0;  
        lastDumpCommand=0;         
        
        if(updateFlag)
        {      
          MainScreen(0,0);  
        }        
    }

    else if (mode && (recordingMode==1) && (recordStartStop==0))
    {      
      if (nextRecord >= (RECORD_DATA_SIZE-6))
      {
        return 1;  
      }
      else
      {
          struct tm * currtime;              
          
          output4Hex((unsigned short) 0x7ff0+(8*UnitsLabel)+AverageOnOff/10);      // debug       
               
          recordData[nextRecord++] = 0x7ff0+(8*UnitsLabel)+AverageOnOff/10;        // overwrites 0x7ffe with rate

          asm("sei");
          safeseconds = seconds;      // disable ints around copying of seconds
          asm("cli");                      
          currtime = gmtime(&safeseconds);  

          recordData[nextRecord++] = 256*currtime->tm_hour + currtime->tm_min;
          recordData[nextRecord++] = 256*currtime->tm_sec + currtime->tm_mon+1;
          recordData[nextRecord++] = 256*currtime->tm_mday + (currtime->tm_year+YEAR0)%100;

          recordStartStop = 1;
          
          if(updateFlag)
          {      
            MainScreen(0,0);  
          }                 
      }
    }
    return 0;
}

/****************************************************************
*addToRecordData     Add data to the recording log
*
*Input:  data     -  number to be added to average
*
*Output: none
*
*Note: Very specific to BlowBy app
****************************************************************/ 
void addToRecordData(float data, unsigned short data2)
{
  short sCFM;
  
  if ((recordingMode==1) && recordStartStop)
  { 
    if (nextRecord < (RECORD_DATA_SIZE-2))    // leave room for final marker
    {     
      // get 16-bit integer value of 100*CFM for recording
      sCFM = (short)(100.0*data);
      //sCFM = nextRecord;                        // debug
      
      output4Hex((unsigned short)sCFM);         // debug       
      output4Hex((unsigned short)data2);         // debug 
             
      recordData[nextRecord++] = sCFM;  
      recordData[nextRecord++] = data2; // RPM 
    }
    else
    {
      (void)startStopRecording(0,menuArrow==0);        // ran into end, stop
    }
  }  
}

/****************************************************************
*doLogDump     Dump recording log in either ASCII or binary
*
*Input:  cmd     -  ASCII_DUMP or BIN_DUMP
*
*Output: none
*
*Note: Very specific to BlowBy app
****************************************************************/ 
void doLogDump(unsigned char cmd)
{
  unsigned short i;
  char str [30];
  short data, tdata;
  float fData;
  char units;
  static float rate, time;    // in case dump is paused and resumed
    
  for (i=firstRecord; i < nextRecord; i++)
  {
    data = recordData[i];
    if (cmd==ASCII_DUMP)
    {     
      if (data >= 0x7ff0)
      {
        struct tm currtime;
        char timestr [26];
        
        tdata = recordData[i+1];
        currtime.tm_hour = (tdata >> 8) & 0xff;
        currtime.tm_min = tdata & 0xff;
        tdata = recordData[i+2]; 
        currtime.tm_sec = (tdata >> 8) & 0xff;
        currtime.tm_mon = (tdata & 0xff) - 1;
        tdata = recordData[i+3]; 
        currtime.tm_mday = (tdata >> 8) & 0xff;
        currtime.tm_year = (tdata & 0xff)+2000-YEAR0;     
        i += 3;
               
        units = (data&0x8) ? 'L' : 'C';      
        data = data&0x7;
        rate = (data) ? (float)data : 0.1;        
        time = 0.0;    
        (void)sprintf(str,"---f=%d%c---\r\n",data,units);
        SCIOutStr(str);
               
        (void)sprintf(timestr,"%02d:%02d:%02d %02d/%02d/%02d",         
        currtime.tm_hour,
        currtime.tm_min,  
        currtime.tm_sec,
        currtime.tm_mon+1,  
        currtime.tm_mday,
        (currtime.tm_year+YEAR0)%100);                                          
        
        (void)sprintf(str,"%s\r\n",timestr);             
      }
      else
      {
        time = time + rate;
        fData = (float)data;
        fData = fData / 100.0;
        data = recordData[++i]; 
        (void)sprintf(str,"%0.1f: %0.2f %d\r\n",time,fData,data);              
      }
      SCIOutStr(str);
    }
    else
    {
      CS=CS + (data & 0xff) + (data>>8);    
      SCIOut16(data);        
    }
    __RESET_WATCHDOG();  // feeds the dog
    if (SCIcommand==FLOW_STOP)
    {
      firstRecord = i+1;
      SCIcommand=0;
      return;   // not break  
    }
  }
  lastDumpCommand = 0;
  firstRecord=0;
  if (cmd==ASCII_DUMP)
  {     
    SCIOutStr("END\r\n");
  }
  else
  {
    data = 0x7ffe;
    CS=CS + (data & 0xff) + (data>>8);    
    SCIOut16(data);   
    SCIOut8(CS); 
  }
}

/****************************************************************
*BlowByAverage     Add data to the average data set
*
*Input:  data     -  number to be added to average
*
*Output: none
*
*Note: Very specific to BlowBy app
****************************************************************/ 
void BlowByAverage(float data,unsigned short data2)
{ 
  if (AverageOnOff)           // AverageOnOff is either 0, or 10,20,30,40,50
  {   
    sumOfCFMReadings += data;
    
    numberReadings++;
    if (numberReadings >= AverageOnOff)
    {
      averageCFM = sumOfCFMReadings / (float)AverageOnOff;    // for display
      addToRecordData(averageCFM,data2);
      
      numberReadings = 0;
      sumOfCFMReadings = 0.0;
    }
  }
  else
  {
    averageCFM = data;         // for display
    addToRecordData(averageCFM,data2);
  }
}

/****************************************************************
*BlowByAverageOnOff     Turn average function on or off
*
*Input:  data  -  0, Off
*                 1 to 5, number of seconds to average over (assumed 10 per second)
*Output: none
*
*Note: Very specific to BlowBy app
*      AverageOnOff  -  number of seconds to 
****************************************************************/ 
void BlowByAverageOnOff(int data)
{
  if((data<0)||(data>5))
  {
    numberReadings = 0;
    sumOfCFMReadings = 0.0;
    AverageOnOff = 0;
    return;
  }
  data *= 10;
  if(data < AverageOnOff)     // if new average < old one, reset
  {          
    numberReadings = 0;
    sumOfCFMReadings = 0.0;
    averageCFM = 0.0;
    AverageOnOff = data;
  }
  else if(data > AverageOnOff)
    AverageOnOff = data;  
}

/****************************************************************
*displayOptions
****************************************************************/     
extern unsigned char recordingMode;
extern unsigned char includeRPM;
extern unsigned char pulsesPerRev2;
extern unsigned char backLightMode;
extern unsigned char backLightTime;
extern unsigned char inactivePwrOffTime;
extern unsigned char lowBatPwrOffTime;
extern unsigned char meterRating;
extern char UnitsLabel;
extern unsigned char filterAverage;
extern float UEK1; 
extern float UEK2;
extern volatile int SquareRootOnOff;
extern float DisplayThres;
extern float OutFactor;

void displayOptions()
{
  char str[30];
    
  switch(option_num)
  {
    case 1:
      (void)sprintf(str,"%d;%d;%d;%d;%d;%d;%d;%d;%d;%d;\r\n",
        recordingMode,includeRPM,pulsesPerRev2,backLightMode,backLightTime,inactivePwrOffTime,lowBatPwrOffTime, 
        meterRating,UnitsLabel,filterAverage);
      SCIOutStr(str);
      break;       
    case 2:   // advanced cal
      (void)sprintf(str,"%1.2f;%1.2f;%d;%1.2f;%1.2f;\r\n",
        UEK1,UEK2,SquareRootOnOff+1,DisplayThres,OutFactor);        
      SCIOutStr(str);    
      break;               
  }
}

/****************************************************************
*setOptions
****************************************************************/     
unsigned short integer, fraction, divisor;
unsigned char noparm;
char* getNum(char *s, unsigned char floattype)
{
  unsigned char state; 
  integer = 0;
  fraction = 0;
  state = 0;
  divisor = 1;
  if (*s == ';')
  {
    noparm = 1;
    return s+1;  
  }
  noparm = 0;
  while(*s && (*s != ';'))
  {
    if ((*s>='0')&&(*s<='9'))
    {
      if (state)
      {
        fraction = 10*fraction + (*s - '0');
        divisor = 10*divisor;
      }
      else
      {
        integer = 10*integer + (*s - '0');        
      }     
      s++;
    }
    else if (floattype && (*s == '.'))
    {
      if (state==0)
      {
        state++;
        s++; 
      }
      else
      {
        return 0;  
      }
    }
    else
    {
      return 0;
    }
  }
  if (*s == ';')
  {
    return s+1;  
  }
  else
  {
    return 0;  
  }
}

unsigned char setOptions1()
{
  char *s;
  
  s = option_str;
  
  s = getNum(s, 0);
  if (s==0)
    return 0;
  if (noparm==0)
  {   
    recordingMode = integer;
  }
  s = getNum(s, 0);
  if (s==0)
    return 0;
  if (noparm==0)
  {   
    includeRPM = integer;
  }
  s = getNum(s, 0);
  if (s==0)
    return 0;
  if (noparm==0)
  {   
    pulsesPerRev2 = integer;
  }    
  s = getNum(s, 0);
  if (s==0)
    return 0;
  if (noparm==0)
  {    
    backLightMode = integer;
  }  
  s = getNum(s, 0);
  if (s==0)
    return 0;
  if (noparm==0)
  {    
    backLightTime = integer;
  }  
  s = getNum(s, 0);
  if (s==0)
    return 0;
  if (noparm==0)
  {    
    inactivePwrOffTime = integer;
  }  
  s = getNum(s, 0);
  if (s==0)
    return 0;
  if (noparm==0)
  {    
    lowBatPwrOffTime = integer;
  }  
  s = getNum(s, 0);
  if (s==0)
    return 0;
  if (noparm==0)
  {    
    meterRating = integer;
  }  
  s = getNum(s, 0);
  if (s==0)
    return 0;
  if (noparm==0)
  {    
    UnitsLabel = integer;
  }  
  s = getNum(s, 0);
  if (s==0)
    return 0;
  if (noparm==0)
  {    
    filterAverage = integer;
  }  
  if (*s=='\0')
  {
    return 1;  
  }
  else
  {
    return 0;
  }                
}

unsigned char setOptions2()
{
  char *s;
  
  s = option_str;
  
  s = getNum(s, 1);
  if (s==0)
    return 0;
  if (noparm==0)
  {  
    UEK1 = (float)integer + (float)fraction/(float)divisor;
  }
  s = getNum(s, 1);
  if (s==0)
    return 0;
  if (noparm==0)
  {  
    UEK2 = (float)integer + (float)fraction/(float)divisor;
  }
  s = getNum(s, 0);
  if (s==0)
    return 0;
  if (noparm==0)
  {  
    SquareRootOnOff = integer-1;  // back to 0-on,1=off
  }
  s = getNum(s, 1);
  if (s==0)
    return 0;
  if (noparm==0)
  {  
    DisplayThres = (float)integer + (float)fraction/(float)divisor;
  }
  s = getNum(s, 1);
  if (s==0)
    return 0;
  if (noparm==0)
  {  
    OutFactor = (float)integer + (float)fraction/(float)divisor;
  }
  if (*s=='\0')
  {
    return 1;  
  }
  else
  {
    return 0;
  }
}

void setOptions()
{
  unsigned char ret;
  
  ret = 0;
  switch(option_num)
  {
    case 1:
      ret = setOptions1();
      break;       
    case 2: 
      ret = setOptions2();
      break;   
  }
  if (ret == 0)
  {
    SCIOutStr("?\r\n"); 
  }
  else
  {
    SCIOutStr("\r\n");  
  }
}
