#include "derivative.h" /* include peripheral declarations */
#include "main.h"

unsigned char EraseEEPROMPage(unsigned char * progAddr, unsigned char numPages)   // 8 bytes per page
{  
  if(FSTAT_FACCERR)
  {
    FSTAT_FACCERR = 1;
  }
  
  while (numPages--)
  { 
    while(!FSTAT_FCBEF);
    // write anywhere to EEPROM buffer
    *progAddr = 1;

    FCMD = mPageErase;    // sector erase
    // clear command buffer empty flag by writing a 1 to it
    FSTAT_FCBEF = 1;    // this launches the command

    WaitUs(4);  		// wait 4 usec
    while(!FSTAT_FCBEF)
    {
      WaitUs(1000);		// wait 1 ms  -- also calls __RESET_WATCHDOG
    }

    if((FSTAT_FACCERR) || (FSTAT_FPVIOL))
    {
      return 1;   // return if error
    }
    while(!FSTAT_FCCF);
    
    progAddr += 8;    // set up for next page
  }
  return 0;
}

unsigned char ProgEEPROM(unsigned char * progAddr, unsigned char * bufferPtr, unsigned short size)
{
  if(FSTAT_FACCERR)
  {
    FSTAT_FACCERR = 1;
  }

  while(size != 0)
  {
    __RESET_WATCHDOG();
    
    while(!FSTAT_FCBEF);
    // write byte to EEPROM buffer
    *progAddr++ = *bufferPtr++;

    FCMD = mBurstProg;    // burst program
    // clear command buffer empty flag by writing a 1 to it
    FSTAT_FCBEF = 1;    // this launches the command

    WaitUs(4);  		// wait 4 usec
    while(!FSTAT_FCBEF)
    {
      WaitUs(20);		// wait 20 usec -- also calls __RESET_WATCHDOG
    }

    if((FSTAT_FACCERR) || (FSTAT_FPVIOL))
    {
      return 1;   // return if error
    }
    size--;
   
  }
  while(!FSTAT_FCCF);     

  return 0;
}  