#include <hidef.h>      /* for EnableInterrupts macro */
#include "derivative.h" /* include peripheral declarations */
#include "main.h"
#include "DAC.h"
#include "LCD.h"
#include "Analog.h"
#include "Math.h"
#include "Menu.h"
#include "Accel.h"
#include "eeprom.h"
#include "OtherBlowByStuff.h"
#include "swirl.h"

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>
#include <time.h>

//Global Variables 
unsigned char SCIData;
unsigned char SCIstate, SCItempcmd, lastDumpCommand;
volatile unsigned char SCIcommand;
unsigned short CS; 
float FixedFloat @ 128;   // 0x80
long FixedLong @ 132;     // 0x84
int FixedInt @ 136;       // 0x88
char FixedChar @ 138;     // 0x8a
char LCDString[LCD_STRING_LENGTH];
signed char X, Y, Z;
signed char lastX, lastY, lastZ;
char SCIstr[30];
unsigned char option_num;
char option_str[30];
unsigned char options_enabled;
extern unsigned char includeRPM;

//IIC
extern volatile char IICData[6];
//
volatile char Buttons=0;
int AN2;
volatile int AN5, AN6;
float FAn2Volts;
float batteryVolts, usbVolts;
static unsigned char batteryCtr=0;

extern unsigned char backLightTime;
extern unsigned char backLightMode; // 1=button, 2=motion, 3=either
short backlightctr;
extern unsigned char inactivePwrOffTime;
short inactivepwroffctr;
extern unsigned char lowBatPwrOffTime;
short lowbatpwroffctr; 

//
//Blowby specific
float K1, DK1, UEK1, K2, DK2, UEK2, UnitsConversion, OutFactor, CFM, CFM2;
char UnitsLabel;
extern const char SpCFM[4];
extern const char SpLPS[4];
extern const char SpRPM[4];
extern const char SpOptions[8];
extern const char SpRezero[7];

const char spCopyright1[17] = {"  Blow By Logger"};
const char spCopyright2[20] = {"Performance Trends,"};
const char spCopyright3[20] = {"               Inc."};
const char spCopyright4[19] = {"(c) Copyright 2013"};  
const char spCopyright5[16] = {"   Version 3.15"};    // version 3.15  02/18/13 
//
//16Hz_isr
char count16Hz=0;
char ctrQsec16Hz=0;
char quarterSecsFlag=0; 
volatile int MainStringLength=0;

//20Hz_isr
unsigned char toggle20Hz = 0; 

// RTC
volatile time_t seconds = 0;
time_t safeseconds;

//Menu
char menuArrow=0, ButtonHeld=0;
int HoldTime=0, HoldTimeOne=15, HoldTimeTwo=80;

void init_globals()
{
  // init in case EEPROM is not initialized 
  set_defaults(); 
    
  SCIstate=0;
  lastDumpCommand=0;
  SCIcommand=0;   
}

void main(void)
{
  unsigned short i;
   
  init();
    
  SOPT1 = 0xE0;       // enable watchdog timer, enable stop
    
  EnableInterrupts;   // enable interrupts        
  
  WaitUs(3000);             //warm up and settle time
  
  AccelInit();
    
  init_globals();
  
//  init_time();    

  init_recordData();
  
  readoptionsfromEEPROM();
   
  // init counters
  backlightctr = 240*backLightTime; 
  lowbatpwroffctr = 4*lowBatPwrOffTime;
  inactivepwroffctr = 240*inactivePwrOffTime;        
  
  LCDInit();                //LCD init
  LCDBlank();
  
  readUSBandBattery();      // read battery voltage into AN6 (since 20 Hz interrupt hasn't been started yet)
  getBattery();             // set up batteryVolts

  if (batteryVolts >= BACKLIGHT_ON_THRESHOLD)
  {              
    NBACKLIGHT = 0;
  }  

  LCDSetCursorPos(1,2);
  LCDStringOut(spCopyright1);
  LCDSetCursorPos(2,2);
  LCDStringOut(spCopyright2);  
  LCDSetCursorPos(3,2);
  LCDStringOut(spCopyright3);    
  LCDSetCursorPos(5,2);
  LCDStringOut(spCopyright4);    
  LCDSetCursorPos(6,2);
  LCDStringOut(spCopyright5);   
  for (i=0; i < 100; i++)   // wait 5 secs
  {
    WaitUs(50000);          // also calls __RESET_WATCHDOG
  }

  NBACKLIGHT = 1;   
   
  K1 = DK1 * UEK1;          //BlowBy init
  K2 = DK2 + UEK2;          // 
  
  SCIOutStr("\r\nHello \r\n");            // SCI is set to 19200 baud
  SCIOutStr("PLEASE SEND A COMMAND\r\n");            
    
  swirl_init();
    
  LCDBlank();   
  MainScreen(0,1);      // also calls Start16Hz 
         
  Start20Hz();
  
  AccelGrabData(&lastX, &lastY, &lastZ);
  backlightctr = 0;
    
  for(;;) 
  {
    __RESET_WATCHDOG();  // feeds the dog
     
    if (quarterSecsFlag)        // every 250 ms 
    {
      unsigned char motion;
           
      AccelGrabData(&X, &Y, &Z);
      //(void)sprintf(msg,"X: %4d  Y: %4d  Z: %4d, %d\r\n",(int)X,(int)Y,(int)Z,backlightctr);  // debug out
      //SCIOutStr(msg);
          
      motion = 0; 
      if (abs(X - lastX) > 10)
      {
        motion = 1;
      } 
      else if (abs(Y - lastY) > 10)
      {
        motion = 1;            
      } 
      else if (abs(Z - lastZ) > 10)
      {
        motion = 1;          
      }
      
      getBattery();         // set up batteryVolts
      
      if (motion)
      {
          if (backLightMode & 2)    // motion
          {
            if (batteryVolts >= BACKLIGHT_ON_THRESHOLD)
            {              
              NBACKLIGHT = 0;
            }
            backlightctr = 240*backLightTime;   // backLightTime in minutes, decr 4 times/sec          
          }
          
          if (inactivePwrOffTime)
          {            
            inactivepwroffctr = 240*inactivePwrOffTime;   // inactivePwrOffTime in minutes, decr 4 times/sec     
          }
      }

      lastX = X;
      lastY = Y;
      lastZ = Z;
      
      //(void)sprintf(SCIstr,"%4d\r\n",ctr++);
      //SCIOutStr(SCIstr);      // test timing -- should count to 100 in 25 seconds     
      
      if (backlightctr) 
      {
        backlightctr--;
        if (backlightctr==0) 
        {
          NBACKLIGHT = 1;            
        }
      }
      
      if (inactivePwrOffTime)
      {            
        inactivepwroffctr--;
        if (inactivepwroffctr==0) 
        {
          powerOffandWaitForKey(0);          
        }             
      }      
      
      if (menuArrow == 0)
      {
        displayBattery();
      }
      
      if (lowBatPwrOffTime)
      {            
        lowbatpwroffctr--;
        if (lowbatpwroffctr==0) 
        {
          powerOffandWaitForKey(0);  
        }             
      } 
           
      quarterSecsFlag = 0;      
    }
    
    if (includeRPM==1)
    {        
      swirl_loop();      
    }
    
    if (SCIcommand)
    {
      unsigned char tempCmd;
      
      tempCmd = SCIcommand;
      SCIcommand = 0;     // so FLOW_STOP command can be recognized during dumps
      
      if (tempCmd==START_REC)
      {
        (void)startStopRecording(1, menuArrow==0);
      }
      else if (tempCmd==STOP_REC)
      {
        (void)startStopRecording(0, menuArrow==0);  
      }
      else if (tempCmd==ERASE_REC)
      {
        init_recordData();
      }                       
      else if ((tempCmd==ASCII_DUMP)||(tempCmd==BIN_DUMP))
      {
        (void)startStopRecording(0,menuArrow==0);   // stop recording
        
        lastDumpCommand = tempCmd;
        firstRecord=0;
        if (tempCmd==BIN_DUMP)
        {          
          CS=0x4c; 
          SCIOut8('&');
          SCIOut8('&'); 
        }
        doLogDump(tempCmd);  
      }
      else if (tempCmd==FLOW_START)
      {
        if (lastDumpCommand)
        {          
          doLogDump(lastDumpCommand);   
        }
      }
      else if (tempCmd==DISPLAY_DATETIME)
      {
        struct tm * currtime;
        char timestr[22];
                      
        asm("sei");
        safeseconds = seconds;      // disable ints around copying of seconds
        asm("cli");                      
        currtime = gmtime(&safeseconds);
        
        (void)sprintf(timestr,"%02d:%02d:%02d %02d/%02d/%02d\r\n",         
        currtime->tm_hour,
        currtime->tm_min,  
        currtime->tm_sec,
        currtime->tm_mon+1,  
        currtime->tm_mday,
        (currtime->tm_year+YEAR0)%100);              
            
        SCIOutStr(timestr);       
      }
      else if (tempCmd==DISPLAY_OPTIONS)
      {
        displayOptions();
      }
      else if (tempCmd==SET_OPTIONS)
      {
        setOptions();
      }
      else if (tempCmd==RESTORE_DEFAULTS)
      {
        set_defaults();
        updateEEPROMfromoptions(0); 
      }
      else if (tempCmd==LOCK_OPTIONS)
      {
        options_enabled = 2;
      }
      else if (tempCmd==UNLOCK_OPTIONS)
      {
        options_enabled = 1;
      }                  
    }

    if(Buttons)
    {
      menu(Buttons);             
      WaitUs(20000);                    
      while(0xF0 & PTBD)      //button is being held
      {
        HoldTime++;                        //
        if(HoldTime == HoldTimeOne)        //Held for short amount of time
          ButtonHeld = 1;                  //
        if(HoldTime == HoldTimeTwo)        //Held for longer time
          ButtonHeld = 2;                  //
        if((ButtonHeld!=0) && (HoldTime%6==0))
        {          
          menu(Buttons);
        }
        
        if (includeRPM==1)
        {        
          swirl_loop();      
        }
              
        WaitUs(20000);        // also calls __RESET_WATCHDOG                 
      }                      
      HoldTime = 0;
      ButtonHeld = 0;
      Buttons = 0;
    }      
  } // loop forever
   
  // please make sure that you never leave main 
}

/****************************************************************
* Battery-related functions
****************************************************************/

void readUSBandBattery()
{
  unsigned char i;
  
  AN5 = 0;
  for (i=0; i < 4; i++)
  {
    AN5 += AnalogRead(USB_ANALOG_CHAN);    // read analog channel 6 (battery) four times and average
  }
  AN5 = AN5 >> 2;

  AN6 = 0;
  for (i=0; i < 4; i++)
  {
    AN6 += AnalogRead(BATTERY_ANALOG_CHAN);    // read analog channel 6 (battery) four times and average
  }
  AN6 = AN6 >> 2; 
}

// copy battery volts from int level and convert to float
void getBattery()
{  
  int tempAN5, tempAN6;
  
  asm("sei");
  tempAN5 = AN5;      // disable ints around copying of battery and USB analog voltages
  tempAN6 = AN6;      // disable ints around copying of battery analog voltage
  asm("cli");
  usbVolts = ((float)AN5)*SUPPLY_VOLTAGE/4095;  
  batteryVolts = ((float)AN6)*SUPPLY_VOLTAGE/4095;
  
  //batteryVolts = 3.39;      // for testing low battery power down
  
  if (lowBatPwrOffTime && (batteryVolts >= LOW_BATTERY_THRESHOLD))   // if 2 bars or more, reset ctr
  {
    lowbatpwroffctr = 4*lowBatPwrOffTime;         // lowBatPwrOffTime in seconds, decr 4 times/sec     
  }
  
  if (batteryVolts < LOW_BATTERY_THRESHOLD)
  {
    NBACKLIGHT = 1;       // turn off backlight to conserve power if one bar or less  
  }
}

void displayBattery()
{
  char battery [8];
  float chargingVolts, chargingThreshold;
  
  (void)strcpy(battery,"[     ]");
  
  if (usbVolts < 0.5)
  {
    chargingVolts = batteryVolts;
    chargingThreshold = 4.05; 
  }
  else
  {
    chargingVolts = usbVolts;
    chargingThreshold = 4.50;      
  }
  
  if (chargingVolts >= chargingThreshold)
  {
    switch (batteryCtr)
    {
      case 0:
        (void)strcpy(battery,"[+    ]");            
        break;
      case 1:
        (void)strcpy(battery,"[ +   ]");            
        break;
      case 2:
        (void)strcpy(battery,"[  +  ]");            
        break;
      case 3:
        (void)strcpy(battery,"[   + ]");            
        break;
      case 4:
        (void)strcpy(battery,"[    +]");            
        break;                                                                     
    }
    batteryCtr++;
    if (batteryCtr > 4)
    {
      batteryCtr = 0; 
    }
  }
  else if (batteryVolts >= 3.70)
  {
    (void)strcpy(battery,"[-----]");            
  }
  else if (batteryVolts >= 3.65)
  {
    (void)strcpy(battery,"[---- ]");            
  }
  else if (batteryVolts >= 3.60)
  {
    (void)strcpy(battery,"[---  ]");            
  }
  else if (batteryVolts >= 3.40)        // LOW_BATTERY_THRESHOLD
  {
    (void)strcpy(battery,"[--   ]");            
  }
  else if (batteryVolts >= 3.20)
  {
    (void)strcpy(battery,"[-    ]");          
  }                         

  LCDSetCursorPos(7,44);
  LCDStringOut(battery);
}

void output4Hex(unsigned short data)      // for debug use only
{
  char msg [20];
                   
  (void)sprintf(msg,"%04x\r\n",data);
  //SCIOutStr(msg);                       // debug
}
  
/****************************************************************
*Init     Used to initialize all the peripherals and variables
*         on the micro. 
* 
*Input: None
*Ouput: None
*
*Note: The timer should be initialized first so that 
*the wait routine can be used in initializing any peripherals that 
*need it
****************************************************************/
void init(void)
{  
  char i;
  // System Clock
  MCGC2 = 0x36;               //
  while(MCGSC_OSCINIT==0);    //
  MCGC1 = 0x87; //0x80;               //
  while(MCGSC_IREFST==1);     //
  while(MCGSC_CLKST!=2);      //select crystal as clock source, 4MHz bus
  
  FCDIV = 19;     // PRDIV8=0, DIV=19: fFCLK = 200 KHz (EEPROM progamming clock) 
  
  // Timer 1 Initialization  
  TPM1MOD = 0x00;                                      
  // TPM1C0SC: CH0F=0,CH0IE=0,MS0B=0,MS0A=0,ELS0B=1,ELS0A=0 - input capture on neg edge
  // actually positive edge, due to inversion by 74HC14
  TPM1C0SC = 0x08; 

  // TPM1C1SC: CH1F=0,CH1IE=0,MS1B=0,MS1A=0,ELS1B=0,ELS1A=1 - input capture on pos edge
  // actually negative edge, due to inversion by 74HC14
  TPM1C1SC = 0x04;                                         

  // TPM1SC: TOF=0,TOIE=1,CPWMS=0,CLKSB=0,CLKSA=1,PS2=0,PS1=1,PS0=0
  TPM1SC   = 0x4A;        // clk source - bus rate clock, TPM clock source divided by 1
                  // so clock input is 4000000/4 = 1 MHz    

  // Timer 3   
     //~Wait function             
  TPM3SC = 0x0A;      //Bus clock (selecting clock source enables timer), prescaler 4 (1 MHz)                           
  TPM3MOD = 0;        //Free running counter
  
  // Timer 2
     //~20Hz interrupt
     //~16Hz interrupt
  TPM2SC = 0x0A;      //Bus clock (selecting clock source enables timer), prescaler 4 (1 MHz)                           
  TPM2MOD = 0;        //Free running counter
     
  // Analog Channels
     //~Pressure Sensors
     //~Analog Ins
  ADCCFG = 0xF5;      //low power, high accuracy, ADC clock = bus/16, 12-bit
  ADCSC2 = 0;         //software trigger           
  APCTL1 = 0x7F;      //A0-A6 are analog pins     
  //ADCSC1 = 0x05     //# of channel to be read, initiates conversion, has conversion complete flag

  // SCI
      //~USB Chip
  //When SCI1BD = 13 BaudRate is 19200      
  SCI1BD = 13;        //Baud Rate is Bus / (16*SCI1BD)
  //SCI1BD = 1;        //250000
  // SCI1C1 = 0;
  SCI1C2 = 0x2C;      //RDRF interrupt enable, transmitter on, receiver on
  //SCI1C3 = 0;
  
  // I2C
     //~DAC
     //~Accel      
  //IIC1A = 0;        //Not DAC (0x4E) or Accel(0x1D). (upper 7 bits)
  IIC1F = 0x17;       //baud rate, initiallizes to 100k
  IIC1C1_IICEN = 1;   //IIC enable
  IIC1S_IICIF = 1;    //Clear any false flags    
  for(i=0;i<6;i++)
  {   
    IICData[i] = 0;   //clear data buffer
  }
     
  // GPIO
     //~LCD Backlight 
     //~Pressure Sensor Power
  PTBD = 0x0E;        //Turn off both pressure sensors and LCD backlight    
  PTBDD = 0x0E;       //(pull ups also do this)
     //~LCD Screen
  PTCDD = 0xFF;       //Set LCD Data bus to outputs
  PTGDD = 0x3C;       //Set LCD Control pins to outputs
  PTFDD = 0x80;       //
  
     //~Push Buttons
  //Already inputs   
  PTBES = 0xF0;       //Detects rising edges
  PTBPS = 0xF0;       //button pins cause interrupts
  PTBSC_PTBACK = 1; //clear any false flag
  PTBSC_PTBIE = 1;  //enable button interrupt
  
  // RTC
  // Configure RTC to interrupt every second
  RTCMOD = 0x11; // interrupt every 16 clocks
  RTCSC = 0x79;  // RTCLKS=3, RTIE=1, RTCPS=9 (32 kHz internal clock 62.5 ms)                                        
}

void disable_peripherals(void)
{                          
  // SCI
  SCI1BD = 0x00;
  SCI1C2 = 0x00; 
  // I2C
  (void)DACOnOff_Out(0x3000);           // high impedance
  WaitUs(20);
  (void)AccelWrite(0x16, 0x00);   // off
  WaitUs(20);
  
  IIC1C1_IICEN = 0; 
  // GPIO  
     //~LCD Screen
  PTCDD = 0x00;
  PTGDD = 0x00;
  PTFDD = 0x00;
  
  // Timer 3   
  TPM3SC = 0x00;                           
  // Timer 2
  TPM2SC = 0x00; 
}

void enable_peripherals(void)
{
  // Timer 3   
  TPM3SC = 0x0A;      //Bus clock (selecting clock source enables timer), prescaler 4 (1 MHz)                           
  // Timer 2
  TPM2SC = 0x0A;      //Bus clock (selecting clock source enables timer), prescaler 4 (1 MHz)                           
  // Analog Channels
  ADCCFG = 0xF5;      //low power, high accuracy, ADC clock = bus/16, 12-bit
  // SCI
  //When SCI1BD = 13 BaudRate is 19200  
  SCI1BD = 13;        //Baud Rate is Bus / (16*SCI1BD)
  //SCI1BD = 1;        //250000
  SCI1C2 = 0x2C;      //RDRF interrupt enable, transmitter on, receiver on  
  // I2C
  IIC1F = 0x17;       //baud rate, initiallizes to 100k
  IIC1C1_IICEN = 1;   //IIC enable 
  
  (void)AccelWrite(0x16, 0x05);
  // DAC will be restored in next 20Hz interrupt
  
  // GPIO
     //~LCD Screen
  PTCDD = 0xFF;       //Set LCD Data bus to outputs
  PTGDD = 0x3C;       //Set LCD Control pins to outputs
  PTFDD = 0x80;       //   
}
  
/****************************************************************
*Wait_us     Waits that number of micro seconds
*
*Input:  data - microseconds to wait
*Output: None
*
*Note: 65536 micro seconds max, unless interrupts are in affect
*      Uses Timer3 Ch0
****************************************************************/  
void WaitUs(int data)
{ 
  TPM3C0V = TPM3CNT + data;   //Set ch0 to current value + data
  TPM3C0SC_CH0F = 0;          //Clear flag
  TPM3C0SC = 0x10;            //Output compare, disconnect from pin
  while(TPM3C0SC_CH0F == 0)   //wait til inerrupt conditions occured
  {
    __RESET_WATCHDOG();       // feeds the dog  
  }
}

/****************************************************************
*Start20Hz     Starts the 1Hz interrupt
*
*Input:  None
*Output: None
*
*Note: 
****************************************************************/  
void Start20Hz(void)
{   
  TPM2C0V = TPM2CNT + 1000;  //Set ch1 to current value + 1000 enought time to finish this Start20Hz call
  TPM2C0SC_CH0F = 0;         //Clear flag
  TPM2C0SC = 0x50;           //Output compare, interrupt enabled, disconnect fron pin   
}

/****************************************************************
*Stop20Hz     Stops the 20Hz interrupt 
*
*Input:  None
*Output: None
*
*Note: 
****************************************************************/  
void Stop20Hz(void)
{ 
  TPM2C0SC_CH0IE = 0;              //disable interrupt 
}

/****************************************************************
*ReStart20Hz     Stops the 20Hz interrupt 
*
*Input:  None
*Output: None
*
*Note: 
****************************************************************/  
void ReStart20Hz(void)
{ 
  TPM2C0SC_CH0IE = 1;              //re-enable interrupt 
}

/****************************************************************
*isr_20Hz     ISR for 20Hz update rate -- every 50 ms
*
*Input:  None
*Output: None
*
*Note:       
****************************************************************/
void __interrupt VectorNumber_Vtpm2ch0 isr_20Hz(void)
{
  float TempVar;
  unsigned char i;
  //unsigned short timer;
  
  TPM2C0SC_CH0F = 0;      //interrupt maintenance
  TPM2C0V += 50000;       //overflowed on 0, now overflow again in 10000 cnts (10 ms = 1/100 sec)
  
  //timer = TPM2CNT;        // to measure time spent in this interrupt
   
  AN2 = 0;
  for (i=0; i < 4; i++)
  {
    AN2 += AnalogRead(2);    // read analog channel 2 four times and average
  }
  AN2 = AN2 >> 2;
  
  FAn2Volts = ((float)AN2)*OLD_SUPPLY_VOLTAGE/4095;              
  
  TempVar = FAn2Volts-K2;
  if(SquareRootOnOff == 0)  //Square root function ON
  {
    i=1;
    if(TempVar < 0)
    {
      TempVar *= -1.0;      // get abs value
      i = -1;
    }                
    CFM = UnitsConversion*K1*SquareRoot(TempVar, 0.001);        
    if(i != 1)
      CFM *= -1.0;          
  }
  else                      //Square root function OFF
    CFM = UnitsConversion*K1*TempVar;                
    
  if(CFM > 50.0)               //limits
    CFM = 50.0;                //
  else if(CFM < -50.0)         //
    CFM = -50.0;               //
          
  TempVar = 4095*CFM*OutFactor/OLD_SUPPLY_VOLTAGE;
  if(TempVar < 0)                   // only output if CFM positive
    TempVar = 0;
  else if(TempVar > 4095)
    TempVar = 4095;        
  i = DACOnOff_Out((int)TempVar);   // output raw reading to DAC every 50 ms     
  
  toggle20Hz = 1 - toggle20Hz;
  
  //output4Hex(TPM2CNT-timer);      // typically 0x3850 -- about 14 ms
  
  if (toggle20Hz == 0)        // every 100 ms
  {
    CFM = (CFM+CFM2)/2.0;     // get average of two readings
    BlowByAverage(CFM,display_rpm);       // and filter/record
    
    readUSBandBattery();      // read USB and battery voltages into AN5 and AN6      
   }
  else
  {
    CFM2 = CFM;    
  }
}

/****************************************************************
*Start16Hz     Starts the 16Hz interrupt
*
*Input:  None
*Output: None
*
*Note: Uses Timer3 Ch2 
****************************************************************/  
void Start16Hz(void)
{   
  TPM2C1V = TPM2CNT + 1000; //Set ch2 to current value + 1000 enough time to finish this Start16Hz call
  TPM2C1SC_CH1F = 0;         //Clear flag
  TPM2C1SC = 0x50;           //Output compare, interrupt enabled, disconnect fron pin   
}

/****************************************************************
*Stop16Hz     Stops the 16Hz interrupt 
*
*Input:  None
*Output: None
*
*Note: Uses Timer3 Ch2
****************************************************************/  
void Stop16Hz(void)
{ 
  TPM2C1SC = 0;              //disable interrupt 
}

/****************************************************************
*isr_16Hz     ISR for 16Hz update rate. Used for LCD
*
*Input:  None
*Output: None
*
*Note: Dissable interrupt for as long as needed to access data 
*      generated by it.  Make this time as short as possible.
*      
****************************************************************/  
void __interrupt VectorNumber_Vtpm2ch1 isr_16Hz(void)
{
  int i;
  float Blowby, tempBB;
  
  TPM2C1SC_CH1F = 0;      //interrupt maintenance
  TPM2C1V += 62500;       //overflowed on 0, now overflow again in 62500 cnts (62.5 ms = 1/16 sec)
    
  ctrQsec16Hz++;
  if(ctrQsec16Hz >= 4)    //This give 0.250 between updates (4/16 = 1/4 = 0.250 sec)
  {
    ctrQsec16Hz = 0;
    quarterSecsFlag = 1;              
  }    
  
  count16Hz++;
  if(count16Hz >= 6)      //This give 0.375s between updates (6/16 = 3/8 = 0.375 sec) 
  {    
    count16Hz = 0;
    
#if 1    
    Blowby = averageCFM; // only calculated in 20Hz interrupt, no collision possible
#else
#if 1    
    Blowby = batteryVolts;    // debug only -- display battery volts
#else    
    Blowby = usbVolts;        // debug only -- display USB volts
#endif
#endif
     
    if(Blowby < 0)             //perform dead band check - get abs value
      tempBB = -Blowby;        //
    else                       //
      tempBB = Blowby;         //
    if(tempBB < DisplayThres)  //
      Blowby = 0;              //
        
    //i = LCDFloatToString(Blowby,LCDString,LCD_STRING_LENGTH,2);
    i = sprintf(LCDString,"%0.2f",Blowby);  
      
    if(menuArrow == 0)
    {      
      i = LCDLengthForBigFont(LCDString);
      if(i < MainStringLength)
      {      
        LCDPageBlank(2);
        LCDPageBlank(3);
        LCDPageBlank(4);
        LCDPageBlank(5);    
        LCDPageBlank(6);
                        
        LCDSetCursorPos(6,CFM_RIGHTX-20);
        if(UnitsLabel == 0)
          LCDStringOut(SpCFM);
        else
          LCDStringOut(SpLPS);
      }
      
      LCDBigStringOut(2,CFM_RIGHTX-i,LCDString);    //output string to LCD
         
      MainStringLength = i;    
    }
    else
    {
      if(i < MainStringLength)
        LCDSectionBlank(0,68,103);             //for when menu is in use
      LCDSetCursorPos(0,104-(i*6)); 
      LCDStringOut(LCDString);
      LCDCharOut(' ');
      if(UnitsLabel == 0)
        LCDStringOut(SpCFM);
      else
        LCDStringOut(SpLPS);
      MainStringLength = i;
    } 
    
  }
  
  if((includeRPM==1)&&(menuArrow == 0))
  {      
    unsigned char lcd_str[8];      
        
    LCDSetCursorPos(6,0);   
    (void)sprintf(lcd_str,"%6ld",display_rpm);
    LCDStringOut(lcd_str);        
    LCDSetCursorPos(6,40);
    LCDStringOut(SpRPM);     
  }
}

/****************************************************************
*isr_Button     interrupt for button presses
*
*Input:  None
*Output: None
*
*Note: 
****************************************************************/
void __interrupt VectorNumber_Vport isr_Button(void)
{
  char newButtons;
  
  PTBSC_PTBACK = 1;   //clear flag
  
  newButtons = 0xF0 & PTBD;  
  if (Buttons==0)
  {
    Buttons = newButtons;
    
  
    if (backLightMode & 1)    // button
    {
      if ((menu_ptr != menu_BlowBy_Default)||(Buttons != BUTTON4))    // not Off button
      {        
        if (batteryVolts >= BACKLIGHT_ON_THRESHOLD)
        {              
          NBACKLIGHT = 0;
        }             
        backlightctr = 240*backLightTime;              // backLightTime in minutes, decr 4 times/sec         
      }
    }
    
    if (inactivePwrOffTime)
    {               
      inactivepwroffctr = 240*inactivePwrOffTime;   // inactivePwrOffTime in minutes, decr 4 times/sec     
    }
  }    
}

/****************************************************************
*isr_rtc     ISR for real time clock (RTC)
*
*Input:  None
*Output: None
*
*Note:       
****************************************************************/
void __interrupt VectorNumber_Vrtc isr_rtc(void)
{
  RTCSC = RTCSC | 0x80;   // clear the interrupt flag
  
  seconds++;              // RTC interrupts every second    
}

/****************************************************************
*SCIOut8     Send one byte of data out through the SCI to the USB 
*            chip
* 
*Input: data to be sent
*Ouput: none
*
*Note: none
****************************************************************/
void SCIOut8(int data)
{
  while(SCI1S1_TDRE==0);               //wait til transmitt buffer is empty
  SCI1D = (char)data;                  //send data to transmitter  
}

/****************************************************************
*SCIOutStr    Send a 0-terminated string to the SCI to the USB 
*             chip.
* 
*Input: data to be sent
*Ouput: none
*
*Note: none
****************************************************************/
void SCIOutStr(char * str)
{  
  while(*str!='\0')
  {
    SCIOut8(*str);
    str++; 
  }
}

/****************************************************************
*SCIOut16     Send two bytes of data out through the SCI to the USB 
*             chip. High byte first, low byte second
* 
*Input: data to be sent
*Ouput: none
*
*Note: none
****************************************************************/
void SCIOut16(int data)
{  
  while(SCI1S1_TDRE==0);               //wait til transmitt buffer is empty
  SCI1D = (char)((data>>8)&0xFF);      //send data 
  while(SCI1S1_TDRE==0);               //wait til transmitt buffer is empty
  SCI1D = (char)(data&0xFF);           //send data
}

/****************************************************************
*SCIIn     Receive data from the USB chip through the SCI
*          currently just echoes character back to terminal
* 
*Input: none
*Ouput: data received
*
*Note: none
****************************************************************/
void __interrupt VectorNumber_Vsci1rx isr_SCIIn(void)
{
  unsigned char data;
  static unsigned char tensdigit, onesdigit, i;
  static unsigned char hour, min, sec, mon, day, year;
  
  if(SCI1S1_RDRF==1)
  {
    data = SCI1D;                        //grab data from receive buffer   
    if (SCIcommand)
    {
      SCIOut8('?');  
    }   
    else
    {
      if (data>=' ')
      {        
        SCIOut8(data);                   //echo data back to terminal
      }
      SCIData = toupper(data);
      switch(SCIstate)
      {
        case 0:     // waiting for first valid char
          if (SCIData=='A')
          {
            SCIstate = 1;
          }
          else if (SCIData=='B')
          {
            SCIstate = 2;
          }
          else if (SCIData=='S')
          {
            SCIstate = 3;
          }
          else if (SCIData=='E')
          {
            SCIstate = 4;
          }
          else if (SCIData=='D')
          {
            SCIstate = 5;
          }
          else if (SCIData=='R')
          {
            SCIstate = 6;
          }
          else if (SCIData=='L')
          {
            SCIstate = 7;
          } 
          else if (SCIData=='U')
          {
            SCIstate = 8;
          }                                          
          else if (SCIData==0x11)  // ctrl-Q
          {
            SCIcommand = FLOW_START;
          }
          else if (SCIData==0x13)  // ctrl-S
          {
            SCIcommand = FLOW_STOP;
          }           
          else
          {
            SCIOutStr("?\r\n");                        
          }
          break;                
        case 1:     // received 'A' (ASCII format)
          if (SCIData=='D')   // AD
          {
            SCItempcmd = ASCII_DUMP;
            SCIstate = 9;
          }
          else
          {
            SCIOutStr("?\r\n");                       
            SCIstate = 0;
          }
          break;
        case 2:     // received 'B' (binary format)
          if (SCIData=='D')   // BD
          {
            SCItempcmd = BIN_DUMP;
            SCIstate = 9;
          }
          else
          {
            SCIOutStr("?\r\n");                       
            SCIstate = 0;
          }
          break;
        case 3:     // received 'S'
          if (SCIData=='D')   // SD
          {
            SCItempcmd = SET_DATETIME;
            SCIstate = 10;    // need to parse datetime          
          }        
          else if (SCIData=='R')   // SR
          {
            SCItempcmd = START_REC;
            SCIstate = 9;          
          }
          else if (SCIData=='T')  // ST
          {
            SCItempcmd = STOP_REC;
            SCIstate = 9;          
          }                             
          else if (SCIData=='O')  // SO
          {
            SCItempcmd = SET_OPTIONS;
            SCIstate = 31;          
          }            
          else
          {
            SCIOutStr("?\r\n");         
            SCIstate = 0;
          }
          break;
        case 4:     // received 'E'
          if (SCIData=='R')   // ER
          {
            SCItempcmd = ERASE_REC;
            SCIstate = 9;          
          }                             
          else
          {
            SCIOutStr("?\r\n");         
            SCIstate = 0;
          }
          break;
        case 5:     // received 'D'
          if (SCIData=='D')   // DD
          {
            SCItempcmd = DISPLAY_DATETIME;
            SCIstate = 9;          
          }
          else if (SCIData=='O')   // DO
          {
            SCItempcmd = DISPLAY_OPTIONS;
            SCIstate = 30;          
          }                                           
          else
          {
            SCIOutStr("?\r\n");         
            SCIstate = 0;
          }
          break;
        case 6:     // received 'R'
          if (SCIData=='D')   // RD
          {
            SCItempcmd = RESTORE_DEFAULTS;
            SCIstate = 9;          
          }                                         
          else
          {
            SCIOutStr("?\r\n");         
            SCIstate = 0;
          }
          break;
        case 7:     // received 'L'
          if (SCIData=='O')   // LO
          {
            SCItempcmd = LOCK_OPTIONS;
            SCIstate = 9;          
          }                                         
          else
          {
            SCIOutStr("?\r\n");         
            SCIstate = 0;
          }
          break;  
        case 8:     // received 'U'
          if (SCIData=='L')   // UL
          {
            SCItempcmd = UNLOCK_OPTIONS;
            SCIstate = 9;          
          }                                         
          else
          {
            SCIOutStr("?\r\n");         
            SCIstate = 0;
          }
          break;                                                    
        case 9:     // received valid command, check if terminated by CR
          if (SCIData=='\r')
          {
            SCIOutStr("\r\n");
            SCIcommand = SCItempcmd;
          }
          else
          {
            SCIOutStr("?\r\n");         
          }          
          SCIstate = 0;
          break;
        case 10:     // parsing datetime - hours (tens)
          if ((SCIData>='0')&&(SCIData<='2'))
          {
            tensdigit = SCIData;
            SCIstate = 11;
          }
          else if (SCIData!=' ')    // optional space (stay in this state)
          {            
            SCIOutStr("?\r\n");         
            SCIstate = 0;                           
          }
          break;
        case 11:     // parsing datetime - hours (ones)
          if ((tensdigit<='1')&&(SCIData>='0')&&(SCIData<='9'))
          {
            onesdigit = SCIData;          
            SCIstate = 12;
          }
          else if ((SCIData>='0')&&(SCIData<='3'))    // (tensdigit=='2')
          {
            onesdigit = SCIData;          
            SCIstate = 12;
          }
          else
          {
            SCIOutStr("?\r\n");         
            SCIstate = 0;
          }
          break;
        case 12:     // parsing datetime - colon
          if (SCIData==':')
          {
            hour = 10*(tensdigit-'0')+(onesdigit-'0');
            SCIstate = 13;
          }
          else
          {
            SCIOutStr("?\r\n");         
            SCIstate = 0;
          }
          break;          
        case 13:     // parsing datetime - minutes (tens)
          if ((SCIData>='0')&&(SCIData<='5'))
          {
            tensdigit = SCIData;
            SCIstate = 14;
          }
          else
          {
            SCIOutStr("?\r\n");         
            SCIstate = 0;
          }
          break;
        case 14:     // parsing datetime - minutes (ones)
          if ((SCIData>='0')&&(SCIData<='9'))
          {
            onesdigit = SCIData;          
            SCIstate = 15;
          }
          else
          {
            SCIOutStr("?\r\n");         
            SCIstate = 0;
          }
          break;          
        case 15:     // parsing datetime - colon
          if (SCIData==':')
          {
            min = 10*(tensdigit-'0')+(onesdigit-'0');
            SCIstate = 16;
          }
          else
          {
            SCIOutStr("?\r\n");         
            SCIstate = 0;
          }
          break;          
        case 16:     // parsing datetime - seconds (tens)
          if ((SCIData>='0')&&(SCIData<='5'))
          {
            tensdigit = SCIData;
            SCIstate = 17;
          }
          else
          {
            SCIOutStr("?\r\n");         
            SCIstate = 0;
          }
          break;
        case 17:     // parsing datetime - seconds (ones)
          if ((SCIData>='0')&&(SCIData<='9'))
          {
            onesdigit = SCIData;          
            SCIstate = 18;
          }
          else
          {
            SCIOutStr("?\r\n");         
            SCIstate = 0;
          }
          break; 
        case 18:     // parsing datetime - space, comma or semicolon          
          if ((SCIData==' ')||(SCIData==',')||(SCIData==';'))
          {         
            SCIstate = 20;
          }
          else
          {
            SCIOutStr("?\r\n");         
            SCIstate = 0;
          }
          break;                   
        case 20:     // parsing datetime - month (tens)
          if ((SCIData>='0')&&(SCIData<='1'))
          {
            sec = 10*(tensdigit-'0')+(onesdigit-'0');
            tensdigit = SCIData;
            SCIstate = 21;
          }
          else if (SCIData!=' ')    // optional space (stay in this state)
          {            
            SCIOutStr("?\r\n");         
            SCIstate = 0;                           
          }
          break;
        case 21:     // parsing datetime - month (ones)
          if ((tensdigit!='1')&&(SCIData>='0')&&(SCIData<='9'))
          {
            onesdigit = SCIData;          
            SCIstate = 22;
          }        
          else if ((SCIData>='1')&&(SCIData<='2'))  // tensdigit=='1'
          {
            onesdigit = SCIData;          
            SCIstate = 22;
          }
          else
          {
            SCIOutStr("?\r\n");         
            SCIstate = 0;
          }
          break;          
        case 22:     // parsing datetime - slash
          if (SCIData=='/')
          {
            mon = 10*(tensdigit-'0')+(onesdigit-'0');
            SCIstate = 23;
          }
          else
          {
            SCIOutStr("?\r\n");         
            SCIstate = 0;
          }
          break;          
        case 23:     // parsing datetime - day (tens)
          if ((mon!=2)&&(SCIData>='0')&&(SCIData<='3'))
          {
            tensdigit = SCIData;
            SCIstate = 24;
          }
          else if ((SCIData>='0')&&(SCIData<='2'))    // mon==2
          {
            tensdigit = SCIData;
            SCIstate = 24;
          }          
          else
          {
            SCIOutStr("?\r\n");         
            SCIstate = 0;
          }
          break;
        case 24:     // parsing datetime - day (ones)
          if ((tensdigit=='0')&&(SCIData>='1')&&(SCIData<='9'))
          {
            onesdigit = SCIData;          
            SCIstate = 25;
          }         
          else if ((tensdigit=='1')&&(SCIData>='0')&&(SCIData<='9'))
          {
            onesdigit = SCIData;          
            SCIstate = 25;
          }
          else if ((tensdigit=='2')&&(SCIData>='0')&&(SCIData<='9'))
          {
            onesdigit = SCIData;          
            SCIstate = 25;
          }                  
        
          else if ((tensdigit=='3')&&(SCIData>='0')&&(SCIData<='1'))
          {
            onesdigit = SCIData;          
            SCIstate = 25;
          }                  
          else
          {
            SCIOutStr("?\r\n");         
            SCIstate = 0;
          }
          break;          
        case 25:     // parsing datetime - slash
          if (SCIData=='/')
          {
            day = 10*(tensdigit-'0')+(onesdigit-'0');
            SCIstate = 26;
          }
          else
          {
            SCIOutStr("?\r\n");         
            SCIstate = 0;
          }
          break;          
        case 26:     // parsing datetime - year (tens)
          if ((SCIData>='0')&&(SCIData<='9'))
          {
            tensdigit = SCIData;
            SCIstate = 27;
          }
          else
          {
            SCIOutStr("?\r\n");         
            SCIstate = 0;
          }
          break;
        case 27:     // parsing datetime - year (ones)
          if ((SCIData>='0')&&(SCIData<='9'))
          {
            onesdigit = SCIData;          
            SCIstate = 29;
          }
          else
          {
            SCIOutStr("?\r\n");         
            SCIstate = 0;
          }
          break;          
        case 29:          
          if (SCIData=='\r')
          {
            struct tm currtime;
            year = 10*(tensdigit-'0')+(onesdigit-'0');
            SCIOutStr("\r\n");
            
            currtime.tm_hour = hour;
            currtime.tm_min = min;
            currtime.tm_sec = sec;
            currtime.tm_mon = mon-1;
            currtime.tm_mday = day;
            currtime.tm_year = year+2000-YEAR0;
            
            seconds = mktime(&currtime);           
          }
          else
          {
            SCIOutStr("?\r\n");         
          }          
          SCIstate = 0;          
          break;          
        case 30:   
          if ((SCIData>='0')&&(SCIData<='9'))
          {
            option_num = SCIData-'0';
            SCIstate = 9;
          }
          else if (SCIData!=' ')    // optional space (stay in this state)
          {            
            SCIOutStr("?\r\n");         
            SCIstate = 0;                           
          }
          break;   
        case 31:   
          if ((SCIData>='0')&&(SCIData<='9'))
          {
            option_num = SCIData-'0';
            i = 0;
            SCIstate = 32;
          }
          else if (SCIData!=' ')    // optional space (stay in this state)
          {            
            SCIOutStr("?\r\n");         
            SCIstate = 0;                           
          }
          break;
        case 32:   
          if (SCIData=='=')
          {
            SCIstate = 33;
          }
          else 
          {            
            SCIOutStr("?\r\n");         
            SCIstate = 0;                           
          }
          break;          
        case 33:
          if (SCIData=='\r')
          {
            // don't output cr/lf here, wait until option_str parsed
            option_str[i] = '\0';
            SCIcommand = SCItempcmd;            
            SCIstate = 0;            
          }
          else
          {
            if ((SCIData>='0')&&(SCIData<='9'))
            {
              option_str[i++] = SCIData; 
              if (i > 29)
              {            
                SCIOutStr("?\r\n");         
                SCIstate = 0;                           
              }
            }
            else if ((SCIData==';')||(SCIData=='.'))
            {
              option_str[i++] = SCIData; 
              if (i > 29)
              {            
                SCIOutStr("?\r\n");         
                SCIstate = 0;                           
              }           
            }
            else
            {            
              SCIOutStr("?\r\n");         
              SCIstate = 0;                           
            }
            break;                      
          }
          break;                                                                      
      }
    }
  }
}

