// based on swirl code

#include <hidef.h>      // for EnableInterrupts macro
#include "derivative.h" // include peripheral declarations

#include <stdio.h>
#include <stdlib.h>     // for abs
#include "swirl.h"

// all vars in upper case were originally bits in 8051 code
unsigned char COUNTS_LOADED, RESET_VALUES;
unsigned char FIRST_PULSE, UPDATE_HEX;
unsigned char DEBUG_ON;

unsigned char which_cnt;
unsigned char *xptr/*, *yptr*/;   // only using one edge for RPM
unsigned char overflow0/*, overflow1*/;
unsigned char display_update;
unsigned short prev_x/*, prev_y*/;
unsigned short startx[QUEUE_SIZE]/*, starty[QUEUE_SIZE]*/;
unsigned short endx[QUEUE_SIZE]/*, endy[QUEUE_SIZE]*/;
long display_rpm;
short current_rpm;
unsigned long current_xsum/*, current_ysum*/;
unsigned long xcnt[QUEUE_SIZE]/*,ycnt[QUEUE_SIZE]*/;
extern unsigned char pulsesPerRev2;  // number of pulses per revolution * 2: 1 to 32 (0.5 to 16)
unsigned char num_readings;

//#define SERIAL_DEBUG__RPM_OUT 1   // uncomment to send RPM value to serial output for debug
//#define SERIAL_DEBUG_OUT	1   // uncomment to send RPM data to serial output for debug
//#define SERIAL_DEBUG_OUT2	1   // uncomment to send RPM data to serial output for debug
char debugstr[40];

void swirl_init(void)
{
	init_swirl_variables();	
	calculate_rpm();    
}

void swirl_loop()    
{     
	if (COUNTS_LOADED)
	{
	  unsigned short overflow[QUEUE_SIZE], reading[QUEUE_SIZE];
	  unsigned short avg_reading, hi_avg_reading, lo_avg_reading;
	  unsigned char ignore[QUEUE_SIZE];
	  unsigned char i;
	  unsigned char overflow_cnt0, overflow_cnt1;
	  unsigned long sum;
	   
    xptr = (unsigned char *)&xcnt;
//    yptr = (unsigned char *)&ycnt;    
    
    for (i = 0; i < QUEUE_SIZE; i++)
		{
		  overflow[i] = *((unsigned short *)xptr)++;
      reading[i] = *((unsigned short *)xptr)++;     
    }
    
    overflow_cnt0 = overflow_cnt1 = 0;
    for (i = 0; i < QUEUE_SIZE; i++)
		{
		  if (overflow[i] != 0)
		  {
		    overflow_cnt1++;  
		  }
		  else
		  {
		     overflow_cnt0++; 
		  }    
    }
    
    if (overflow_cnt0 > overflow_cnt1)
    {
      xptr = (unsigned char *)&xcnt;
      for (i = 0; i < QUEUE_SIZE; i++)
		  {
			  *(((unsigned short *)xptr)++) = 0;
			  ((unsigned short *)xptr)++;        
      }
    }
    
    sum = 0;
    for (i = 0; i < QUEUE_SIZE; i++)
		{
		  sum += reading[i];    
    }
    avg_reading = sum / QUEUE_SIZE;    
    hi_avg_reading = avg_reading + avg_reading/10;   // plus 10%
    lo_avg_reading = avg_reading/2;
    
    num_readings = 0;
    for (i = 0; i < QUEUE_SIZE; i++)
		{
		  if (reading[i] > hi_avg_reading)
		  {
		    ignore[i] = 1;  
		  }
		  else if (reading[i] < lo_avg_reading)
		  {
		    ignore[i] = 1;  
		  }				  		  
		  else
		  {
		    ignore[i] = 0;
		    num_readings++;
		  }
    } 

#if 0    
    xptr = (unsigned char *)&xcnt;
    for (i = 0; i < QUEUE_SIZE; i++)
		{
		  overflow[i] = *((unsigned short *)xptr)++;
      reading[i] = *((unsigned short *)xptr)++;
#ifdef SERIAL_DEBUG__RPM_OUT	 
  sprintf(debugstr,"%02x%04x %d\r\n",overflow[i],reading[i],ignore[i]);
  SCIOutStr(debugstr);  
#endif               
    }
     __RESET_WATCHDOG();    
#endif     

    current_xsum = /*current_ysum =*/ 0;
    xptr = (unsigned char *)&xcnt;    
    for (i = 0; i < QUEUE_SIZE; i++)
		{
		  if (ignore[i]==0)
		  {		    
      current_xsum += *(((unsigned long *)xptr)++);
//      current_ysum += *(((unsigned long *)yptr)++);
		  }
		  else
		  {
		    ((unsigned long *)xptr)++;
		  }
    }

    if (num_readings)
    {
      
      calculate_rpm();
      
      if (UPDATE_HEX)
      { 
        display_rpm = current_rpm;                                                 	        
        UPDATE_HEX = 0;  
      }
    }
    which_cnt = 0;
    xptr = (unsigned char *)&xcnt;
 //   yptr = (unsigned char *)&ycnt;
    FIRST_PULSE = 1;
    COUNTS_LOADED = 0;         
    TPM1C0SC_CH0IE = 1;				// Trigger off next Positive Edge
	}

	if (RESET_VALUES)
	{
      RESET_VALUES = 0;
      COUNTS_LOADED = 1;
      FIRST_PULSE = 1;
      which_cnt = 0;
      overflow0 = 0;
//      overflow1 = 0;
      xptr = (unsigned char *)&xcnt;
//      yptr = (unsigned char *)&ycnt;
      COUNTS_LOADED = 0;
      TPM1C0SC_CH0IE = 1;     // Signal Input, Capture Positive, Enable IRQ

      current_xsum = /*current_ysum =*/ 0;
	}
	else if (overflow0 /*+ overflow1*/ > MIN_RPM_OVERFLOW)
	{
	    display_rpm = 0;
   		RESET_VALUES = 1;
	}

   __RESET_WATCHDOG();
}    

void calculate_rpm(void)
{
  //unsigned char start;
  unsigned short utmp, limit;//, utmp2, utmp3;
  //short tmp;
  unsigned long ultmp;//, ultmp2;
  
  // at pulsesPerRev2==1 (0.5 ppr), 84 (120*.7) rpm is lowest rpm that can be displayed
  // at pulsesPerRev2==30 (15 ppr), 3 (4*.7) rpm is lowest rpm that can be displayed
  // in general, min rpm = 42/ppr

  //original code: ultmp = CLOCK_RATE * 60 * NUM_PPR / (current_xsum * QUEUE_SIZE);
  ultmp = CLOCK_RATE * 60 * num_readings / current_xsum;
  utmp = (unsigned short) (2L*ultmp / (long)pulsesPerRev2);
  if (utmp > 9999) utmp = 9999;
  
#if 0  
  tmp = utmp;

  ultmp = current_xsum / QUEUE_SIZE;		// Avg Pulses per Edge
  ultmp2 = ultmp - (ultmp >> 3);			// 12.5% Deviation
  	
  xptr = (unsigned char *)&xcnt;
  yptr = (unsigned char *)&ycnt;
  for (start = 0, utmp2 = utmp3 = 0; start < QUEUE_SIZE; start++)
  {

  	if (ultmp2 < *(((unsigned long *)xptr)++)) utmp2++;
  	if (ultmp2 < *(((unsigned long *)yptr)++)) utmp3++;
  }
  if (utmp2 < utmp3) tmp = -tmp;
#endif

  if ((current_rpm > 3) && (utmp <= 3))
  {
    return;
  }
  
  limit = 7*(120/pulsesPerRev2)/10;
  if (utmp < limit)
  {
    utmp = 0;  
  }

#ifdef SERIAL_DEBUG__RPM_OUT	 
  sprintf(debugstr,"RPM: %d\r\n",utmp);
  SCIOutStr(debugstr);  
#endif

	if (--display_update == 0) 
	{
      if (utmp > 5000) display_update = 8;
      else if (utmp > 2000) display_update = 5;
      else if (utmp > 500) display_update = 2;
      else display_update = 1;

      current_rpm = utmp;  // was tmp;

	    UPDATE_HEX = 1;
  }
  __RESET_WATCHDOG();  
}

void init_swirl_variables(void)
{
  unsigned char i;
  
	DEBUG_ON = 0;

	// clear all pulse counts
	for (i = 0; i < QUEUE_SIZE; i++) 
	{
		xcnt[i] = 0x0;
		//ycnt[i] = 0x0;
 	}
 	
  which_cnt = 0; 
  display_update = 1;

  COUNTS_LOADED = 0;
  FIRST_PULSE = 0;
  RESET_VALUES = 1;

  current_rpm = 0;
  display_rpm = 0L;
   
  xptr = (unsigned char *)&xcnt;
//  yptr = (unsigned char *)&ycnt;
  current_xsum = /*current_ysum =*/ 0;

  overflow0 = /*overflow1 = */ 0x00;
}

// interrupt on negative edge (really, positive edge due to inversion)
void __interrupt VectorNumber_Vtpm1ch0 Vtpm1ch0_isr(void)
{
    unsigned short utmp;

    TPM1C0SC_CH0F = 0;    // ack channel interrupt        

	if (!COUNTS_LOADED) 
	{
		utmp = ((unsigned short)TPM1C0VH << 8) | TPM1C0VL;
		if (FIRST_PULSE) 
		{
		    TPM1C1SC_CH1F = 0;      // reset if set
            TPM1C1SC_CH1IE = 1;
			prev_x = utmp;
  			overflow0 = 0;
			if (DEBUG_ON) startx[which_cnt] = prev_x;
		}
		else 
		{
			if (DEBUG_ON) endx[which_cnt] = utmp;
#ifdef SERIAL_DEBUG_OUT2			
		  sprintf(debugstr,"utmp=%04x, prev_x=%04x\r\n",utmp,prev_x);
		  SCIOutStr(debugstr); 
#endif			
			if (prev_x > utmp) 
			{
			  overflow0--;
#ifdef SERIAL_DEBUG_OUT			
		    sprintf(debugstr,"overflow0- %d\r\n",overflow0);
		    SCIOutStr(debugstr); 
#endif			  
			}
			prev_x = utmp - prev_x;

			*(((unsigned short *)xptr)++) = overflow0;
			*(((unsigned short *)xptr)++) = prev_x;
			
#ifdef SERIAL_DEBUG_OUT			
		  sprintf(debugstr,"%04x%04x\r\n",overflow0,prev_x);
		  SCIOutStr(debugstr); 
#endif

			prev_x = utmp;
#ifdef SERIAL_DEBUG_OUT2			
		  sprintf(debugstr,"prev_x=%04x\r\n",prev_x);
		  SCIOutStr(debugstr); 
#endif			
			overflow0 = 0;

			if (DEBUG_ON) 
			{
				if (which_cnt + 1 < QUEUE_SIZE) startx[which_cnt + 1] = prev_x;
			}
		}
	}        
}

// interrupt on positive edge (really, negative edge due to inversion)
void __interrupt VectorNumber_Vtpm1ch1 Vtpm1ch1_isr(void)
{
    unsigned short utmp;

    TPM1C1SC_CH1F = 0;    // ack channel interrupt
    
//	utmp = ((unsigned short)TPM1C1VH << 8) | TPM1C1VL;
	if (FIRST_PULSE) 
	{	
//		prev_y = utmp;
//		overflow1 = 0;
//		if (DEBUG_ON) starty[which_cnt] = prev_y;
		FIRST_PULSE = 0;
	}
	else 
	{
#if 0
		if (DEBUG_ON) endy[which_cnt] = utmp;
		if (prev_y > utmp) overflow1--;
	    prev_y = utmp - prev_y;

		*(((unsigned short *)yptr)++) = overflow1;
		*(((unsigned short *)yptr)++) = prev_y;
#endif
		if (++which_cnt >= QUEUE_SIZE) 
		{
	   	   	FIRST_PULSE = 1;
   			COUNTS_LOADED = 1;
            TPM1C1SC_CH1IE = 0;
            TPM1C0SC_CH0IE = 0; // added
            
#ifdef SERIAL_DEBUG_OUT	            
  		  SCIOutStr("COUNTS LOADED\r\n"); 
#endif  		  
            
		}
		else 
		{
//			prev_y = utmp;
//			overflow1 = 0;
//			if (DEBUG_ON) starty[which_cnt] = prev_y;
		}
	}
}

void __interrupt VectorNumber_Vtpm1ovf Vtpm1ovf_isr (void) 
{
    if (TPM1SC_TOF)
    {
        TPM1SC_TOF = 0;  // Clear the interrupt flag
        
		  if (!COUNTS_LOADED)
		  {
  			if (!TPM1C0SC_CH0F) 
  			{ 			  
  			  overflow0++;        // if no pending edge, this is an overflow
#ifdef SERIAL_DEBUG_OUT	            
  		    sprintf(debugstr,"overflow0+ %d\r\n",overflow0);
  		    SCIOutStr(debugstr);  
#endif  			  
  			}
  			if (!TPM1C1SC_CH1F) 
  			{  			  
//  			  overflow1++;        // if no pending edge, this is an overflow			  
  			}
   	 	}        
    }
}                       
